SystemOrganization addCategory: #'SOUL-GrammarTerms'!

Text subclass: #DownedQuotedCode
	instanceVariableNames: 'content block variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!DownedQuotedCode commentStamp: '<historical>' prior: 0!
Represents SOUL QuotedCode on the Smalltalk level. It's a subclass of text containing information an behaviour to reproduce an upped version of the quotedcode in SOUL
!

!DownedQuotedCode class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
fromString: aString
	
	| qc |
	qc := QuotedCodeTerm source: aString.
	^self
		fromString: aString
		variables: qc variablesIncludingNils
		content: qc content
		block: qc block! !

!DownedQuotedCode class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
fromString: aString variables: variables content: terms block: aBlock
	
	| instance |
	instance := super fromString: aString.
	instance variables: variables content: terms block: aBlock.
	^instance! !

!DownedQuotedCode methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asList
	
	^SoulFactory current makeList
		termSequence:
			(TermSequence new terms: (content collect: [:each | each quotedCodeToList]))! !

!DownedQuotedCode methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"Print the string of the downed quotedcode"

	
	aStream print: string! !

!DownedQuotedCode methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulUp
	"produce an upped quotedcode of this downed quotedcode"

	
	^QuotedCodeTerm
		source: self asString
		variables: variables
		content: content
		block: block! !

!DownedQuotedCode methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
variables: vars content: terms block: aBlock
	
	variables := vars.
	content := terms.
	block := aBlock! !

DownedQuotedCode subclass: #DownedTemplateQuery
	instanceVariableNames: 'type queries trees'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!DownedTemplateQuery class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
fromString: aString variables: aCollectionOfVars content: aContent block: aBlock parsetrees: aParsetree queries: aQueries type: aType
	
	| inst |
	inst := super fromString: aString variables: aCollectionOfVars content:
			aContent block: aBlock.
	inst parsetree: aParsetree queries: aQueries type: aType.
	^inst! !

!DownedTemplateQuery methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
parsetree
	
	^trees! !

!DownedTemplateQuery methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
parsetree: aParsetree queries: aQueries type: aType
	
	trees := aParsetree.
	queries := aQueries.
	type := aType! !

!DownedTemplateQuery methodsFor: 'reification' stamp: 'AndyKellens 11/2/2011 09:53'!
soulUp
	"produce an upped quotedcode of this downed quotedcode"

	
	^SoulFactory current makeTemplateQuery
		source: self asString
		variables: variables
		content: content
		block: block
		parsetrees: trees
		queries: queries
		type: type! !

Exception subclass: #UnboundVariableException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

Object subclass: #AbstractTerm
	instanceVariableNames: 'definitionRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!AbstractTerm commentStamp: '<historical>' prior: 0!
The superclass of all parsetree elements that represent SOUL logic terms.!

!AbstractTerm class methodsFor: 'signal constants' stamp: 'sp 24/10/11 15:26'!
errorSignal
	
	^SoulError! !

!AbstractTerm class methodsFor: 'open unification pragmas' stamp: 'sp 24/10/11 15:26'!
openUnificationMethodPragmas
	
	<pragmas: #instance>
	^#(#unificationmethod:)! !

!AbstractTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"Every term can be visited by a visitor and should implement this part of the design pattern

	Note that the accept method must return the value of the visit message."

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
additionalClausesForDefinitionRepository
	
	^nil! !

!AbstractTerm methodsFor: 'fuzzy' stamp: 'AndyKellens 11/2/2011 09:53'!
annotation
	 
	^SoulFactory current defaultAnnotation! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asCompilationString
	
	| tempStream |
	tempStream := WriteStream on: String new.
	self printForCompileOn: tempStream.
	^tempStream contents! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asDCGConclusionWithSize: aSize
	
	self error: 'This term cannot be used as a head in a DCG rule'! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asDCGConditionFor: counter
	
	self error: 'This term cannot be used as a production in a DCG rule'! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateName
	" send by repository when clauses are added to it. At that time, no runtime environment is available"

	
	^self asPredicateNameInEnv: nil startAt: nil! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: anIndex
	"When this terms is used as a predicate (functor), generate its representation as a predicatename. The default behaviour generates an error"

	
	self error: 'The term ' , self asString , ' cannot be used as a predicate'! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
asPrettyString
	
	| aStream |
	aStream := TextStream on: String new.
	self prettyPrintOn: aStream scope: #asClause.
	^aStream contents! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asProlog
	
	^self asPrologIn: nil startAt: nil! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPrologIn: env startAt: idx
	
	| stream |
	stream := WriteStream on: String new.
	self printAsPrologOn: stream scope: #asClause in: env startAt: env.
	^stream contents! !

!AbstractTerm methodsFor: 'quotedCode' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	"This method is used by the downing process for the QuotedCode. Basically, the parser generates a list of constants and variables when parsing the text inside the QuotedCode. When the quotedcode is downed, it needs to be converted to a string. This is where this method gets invoked"

	
	self halt.
	^self asString! !

!AbstractTerm methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
definitionRepository
	"Each term keeps a pointer to the repository where it was defined. This method is used by the resolution process to determine the term's static scoped repository"

	
	^definitionRepository! !

!AbstractTerm methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
definitionRepository: aRepository
	"Each term keeps a pointer to the repository where it was defined"

	
	definitionRepository := aRepository! !

!AbstractTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
explicitSoulUp
	"This message is used to explicitly wrap a Soul term anyhow. It is normally called within a smalltalkterm in a logic program. For special purposes only, to be used with caution"

	
	^super soulUp! !

!AbstractTerm methodsFor: 'fuzzy' stamp: 'sp 24/10/11 15:26'!
innerTerm
	
	^self! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isAnnotatedTerm
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCallTerm
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCompoundTerm
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isConstantFunctor
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCut
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isHiddenVariable
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isIndexable
	
	^true! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isKeywordFunctor
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isMessageFunctor
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isNative
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToCompoundTerm: aCompoundTerm
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToEmptyList: anEmptyListConstant
	
	^false! !

!AbstractTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToSmalltalkTerm: aTerm
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToSymbiosisTerm: aSymbiosisTerm
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToTermSequence: aTermSequence
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToUppedObject: anUppedObject
	
	^false! !

!AbstractTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToVariable: aVariable
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUnaryMessageFunctor
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUppedObject
	
	^false! !

!AbstractTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isVariable
	
	^false! !

!AbstractTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^0! !

!AbstractTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
predicate
	"For storing this kind of term in the logic database, this method creates a unique key to identify this term when the resolution process needs to retrieve it 	from the logic database. The default behaviour is generates an error."

	
	^self
		error:
			'Cannot store a term with as conclusion a ' , self class asString ,
					' in the logic repository'! !

!AbstractTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: anIndex
	"This method is called at runtime if the lexicalAddress for this clause was not filled in. This can either be because this clause was not present when the database was 	compiled, e.g. if a variable was used as functor"

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintAsFunctorOn: aStream scope: id
	"This method should print the textual representation of the term when it  
	is used as a functor. This is necessary if uppedobjects are used as  
	functors"

	
	self prettyPrintOn: aStream scope: id! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintAsKeywordArgOn: aStream scope: id
	
	^self prettyPrintOn: aStream scope: id! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintAsKeywordOn: aStream scope: id
	"This method should print the textual representation of the term when it  
	is used as a keyword. This is necessary if uppedobjects are used as  
	keyword"

	
	self prettyPrintOn: aStream scope: id! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	"pretty printing"

	
	self printOn: aStream! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsFunctorOn: aStream
	"This method should print the textual representation of the term when it  
	is used as a functor. This is necessary if uppedobjects are used as  
	functors"

	
	self printOn: aStream! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsKeywordArgOn: aStream
	
	^self printOn: aStream! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologFunctorOn: aStream scope: id in: env startAt: idx
	"This method should print the textual representation of the term when it  
	is used as a functor. This is necessary if uppedobjects are used as  
	functors"

	
	self printAsPrologOn: aStream scope: id in: env startAt: idx! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: anObject in: env startAt: idx
	
	aStream write:
			'error("Translation not yet implemented, please contact KrisG.")'	"self subclassResponsibility"! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"This method should print a representation of the term which is  
	SmallTalk code that constructs the term"

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"This method should print the textual representation of the term"

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
printOrderedCollectionForCompile: collection on: aStream
	
	aStream write: 'OrderedCollection new'.
	collection
		do:
			[:element | 
			aStream write: 'add:('.
			element printForCompileOn: aStream.
			aStream write: ')']
		separatedBy: [aStream write: ';'].
	aStream write: ';yourself'! !

!AbstractTerm methodsFor: 'quotedCode' stamp: 'sp 24/10/11 15:26'!
quotedCodeToList
	" To extract the list representation of a quotedcode. This method gets called to all elements in a quotedcode before they are put inside the list representation. All normal terms inside the quotedcode should simply return themselves"

	
	^self! !

!AbstractTerm methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	"Each term keeps a pointer to the repository where it was defined. This method is used by the repository when a clause (and all its subterms) get added to it"

	
	self definitionRepository: aRepository! !

!AbstractTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: anEnv startAt: anIndex
	"This method gets invoked by the kernel to start a resolution process with this term as a goal"

	
	self error: 'Cannot resolve a ' , self class asString! !

!AbstractTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockString
	"This method is used by the building process for the SmalltalkTerms. Basically, the parser generates a list of constants and variables when parsing the 	code inside the SmalltalkTerm. When the code is handed to the Smalltalk Compiler, the code needs to be printed out again. This is where this method 	gets invoked"

	
	^self asString! !

!AbstractTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"downing a SOUL term to the SmallTalk Environment means returning the SOUL term itself"

	
	^self! !

!AbstractTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulUp
	"upping a SOUL term to the SOUL Environment means returning  the SOUL term itself"

	
	^self! !

!AbstractTerm methodsFor: 'fuzzy' stamp: 'sp 24/10/11 15:26'!
term
	
	^self! !

!AbstractTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	"This method should be implemented by the subclasses and return a fully ground term representation for this term by walking the environment when 	necessary and constructing a new term for the representation as a result"

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'code tagging' stamp: 'sp 24/10/11 15:26'!
ugly: message
	"Used to tag code with annotations about how ugly it is. Implemented here because it is used in the keyworded syntax extension but does not actually seem to exist (When I developed the keyworded syntax I could use that message but it does not seem to be part of the base visualworks packages, I'm guessing it came from the StarBrowser or something"
	
	! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"double dispatch the unification of the term"

	
	^self subclassResponsibility! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithAnnotatedTerm: aTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"Simply ignore the annotation part.
	Otherwise, crisp goals will not resolve against annotated rules."

	"^self
		unifyWith: aTerm transitiveTerm
		inEnv: anEnv
		myIndex: sourceIndex
		hisIndex: targetIndex
		inSource: inSource"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a compound, standard behaviour is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCut: aCut inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithDelayedVariable: aVariable inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"standard behaviour of unification with a delayedvariable is the same as with a normal variable"

	
	^self unifyWithVariable: aVariable inEnv: anEnv myIndex: sourceIndex
		hisIndex: targetIndex inSource: inSource! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self unifyWithUppedObject: aTerm inEnv: anEnv myIndex: myIndex hisIndex:
			hisIndex inSource: inSource! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithKeywordFunctor: term inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self unifyWithCompoundTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex:
			hisIndex inSource: inSource! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithMessageFunctor: term inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithNativeClause: aNative inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithNegVariable: aVariable inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a positive variable, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithPosVariable: aVariable inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a positive variable, standard is as with a normal variable"

	
	^self unifyWithVariable: aVariable inEnv: anEnv myIndex: sourceIndex
		hisIndex: targetIndex inSource: inSource! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a quotedCodeTerm, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithResultMessageTerm: aRM inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: aSTTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a smalltalkterm, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithTemplateQueryTerm: aTQCTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a typedquotedCodeTerm, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithTermSequence: term inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUnaryMessageFunctor: unaryFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self = unaryFunctor message! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUnderscoreVariable: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"everything unifies with an underscore variable"

	
	^true! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with an object, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a varargscompound, standard behaviour is false"

	
	^false! !

!AbstractTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"unify this term with a variable, standard is false"

	
	^false! !

!AbstractTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	"This method is used by repositories to look up all predicated used inside a clause. Standard behaviour returns false"

	
	^false! !

AbstractTerm subclass: #MultiPartFunctor
	instanceVariableNames: 'keywords'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

MultiPartFunctor subclass: #KeywordFunctor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!KeywordFunctor methodsFor: 'comparing' stamp: 'sp 24/10/11 15:26'!
= anObject
	
	^(anObject isKindOf: KeywordFunctor)
		ifTrue: [anObject keywords = keywords]
		ifFalse: [super = anObject]! !

!KeywordFunctor methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	
	^aVisitor keywordFunctorVisit: self! !

!KeywordFunctor methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isKeywordFunctor
	
	^true! !

!KeywordFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^keywords size! !

!KeywordFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: otherTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^otherTerm
		unifyWithKeywordFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

!KeywordFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithKeywordFunctor: otherKeyFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	self multiplicity = otherKeyFunctor multiplicity ifFalse: [^false].
	self keywords
		with: otherKeyFunctor keywords
		do:
			[:myKey :otherKey | 
			(myKey unifyWith: otherKey inEnv: anEnv myIndex: myIndex hisIndex: hisIndex
				inSource: inSource) ifFalse: [^false]].
	^true! !

!KeywordFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aVariable
		unifyWithKeywordFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

MultiPartFunctor subclass: #MessageFunctor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!MessageFunctor methodsFor: 'comparing' stamp: 'sp 24/10/11 15:26'!
= anObject
	
	^(anObject isKindOf: MessageFunctor)
		ifTrue: [anObject keywords = keywords]
		ifFalse: [super = anObject]! !

!MessageFunctor methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	
	^aVisitor messageFunctorVisit: self! !

!MessageFunctor methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isMessageFunctor
	
	^true! !

!MessageFunctor methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
messageSelector
	"UGLY: this assumes the keyword is a constant, it might also be a variable ..."

	
	^(keywords
		inject: ''
		into: [:concatenated :keyword | concatenated , keyword name , ':'])
		asSymbol! !

!MessageFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^keywords size + 1! !

!MessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: otherTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^otherTerm unifyWithMessageFunctor: self inEnv: anEnv myIndex: hisIndex
		hisIndex: myIndex inSource: inSource! !

!MessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithMessageFunctor: otherMessageFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	self multiplicity = otherMessageFunctor multiplicity ifFalse: [^false].
	self keywords
		with: otherMessageFunctor keywords
		do:
			[:myKey :otherKey | 
			(myKey unifyWith: otherKey inEnv: anEnv myIndex: myIndex hisIndex: hisIndex
				inSource: inSource) ifFalse: [^false]].
	^true! !

!MessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithMessageFunctor: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!MessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aVariable
		unifyWithMessageFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

!MultiPartFunctor class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
keyword: keyword
	
	^self new keyword: keyword! !

!MultiPartFunctor class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
keywords: keywords
	
	^self new keywords: keywords! !

!MultiPartFunctor class methodsFor: 'instance creation' stamp: 'AndyKellens 11/2/2011 09:53'!
name: aName
	
	^self
		keywords:
			(aName asSymbol keywords
				collect:
					[:keywordName | SoulFactory current makeConstant name: (keywordName allButLast: 1) asSymbol])! !

!MultiPartFunctor methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addKeyword: keyword
	
	keywords addFirst: keyword! !

!MultiPartFunctor methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: idx
	"UGLY: the recursive sending of asPredicateNameInEnv might be better replaced with a new message, asKeywordInEnv ..."

	
	^self keywords
		inject: ''
		into:
			[:predicateName :keyword | predicateName , (keyword asPredicateNameInEnv: anEnv startAt: idx) , ':']! !

!MultiPartFunctor methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isIndexable
	
	self keywords
		detect: [:keyword | keyword isKindOf: Variable]
		ifNone: [^true].
	^false! !

!MultiPartFunctor methodsFor: 'initializing' stamp: 'sp 24/10/11 15:26'!
keyword: keyword
	
	keywords := OrderedCollection with: keyword! !

!MultiPartFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
keywords
	
	^keywords! !

!MultiPartFunctor methodsFor: 'initializing' stamp: 'sp 24/10/11 15:26'!
keywords: keys
	
	keywords := keys! !

!MultiPartFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	self subclassResponsibility! !

!MultiPartFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
name
	
	^(keywords
		inject: ''
		into: [:concatenated :keyword | concatenated , keyword name , ':'])
		asSymbol! !

!MultiPartFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologFunctorOn: aStream scope: id in: env startAt: idx
	"TODO: What if any of the keywords are variables?"

	
	self keywords
		do: [:keyword | aStream write: keyword name]
		separatedBy: [aStream write: '_']	"Clearer without the last underscore, but potential ambiguity between unary messages and keyword messages with one argument. Uncomment if unambigous translation is needed"
	"aStream write: '_'."! !

!MultiPartFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' keywords: (OrderedCollection new '.
	self keywords
		do:
			[:keyword | 
			aStream write: 'add:('.
			keyword printForCompileOn: aStream.
			aStream write: ')']
		separatedBy: [aStream write: ';'].
	aStream write: ';yourself)'! !

!MultiPartFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	keywords
		do:
			[:keyword | 
			keyword printOn: aStream.
			aStream write: ':']! !

!MultiPartFunctor methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	"see comment in AbstractTerm"

	
	self definitionRepository: aRepository.
	keywords
		do: [:keyword | keyword recursiveDefinitionRepository: aRepository]! !

!MultiPartFunctor methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self class
		keywords:
			(keywords
				collect:
					[:keyword | keyword transitiveLookupIn: anEnv startAt: anIndex])! !

AbstractTerm subclass: #ResolvableTerm
	instanceVariableNames: 'lexicalAddress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

ResolvableTerm subclass: #AnnotatedTerm
	instanceVariableNames: 'term annotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!AnnotatedTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
term: aTerm annotation: anAnnotationTerm
	
	^(super new)
		term: aTerm annotation: anAnnotationTerm;
		yourself! !

!AnnotatedTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	
	^aVisitor visitAnnotatedTerm: self! !

!AnnotatedTerm methodsFor: 'annotated terms' stamp: 'sp 24/10/11 15:26'!
annotation
	
	^annotation! !

!AnnotatedTerm methodsFor: 'delegation' stamp: 'sp 24/10/11 15:26'!
doesNotUnderstand: aMessage
	
	self halt.
	^term
		perform: aMessage selector
		withArguments: aMessage arguments! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
functor
	
	^self term functor! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
innerTerm
	
	^term innerTerm! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isAnnotatedTerm
	
	^true! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCallTerm
	
	^false	"	^ term isCallTerm"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isConstantFunctor
	
	^false	"^ term isConstantFunctor"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isIndexable
	
	^false	"^ term isIndexable"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isKeywordFunctor
	
	^false	"^ term isKeywordFunctor"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isMessageFunctor
	
	^false	"
	^ term isMessageFunctor"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isNative
	
	^false	"^ term isNative"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUnaryMessageFunctor
	
	^false	"^ term isUnaryMessageFunctor"! !

!AnnotatedTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isVariable
	
	^false	"^ term isVariable"! !

!AnnotatedTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
lexicalAddressInEnv: anEnv startAt: anIndex ifAbsent: aBlock
	
	^super lexicalAddressInEnv: anEnv startAt: anIndex ifAbsent: aBlock	"^term lexicalAddressInEnv: anEnv startAt: anIndex ifAbsent: aBlock"! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^term multiplicity! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
predicate
	"Necessary in order to store clauses with annotated head in repository."

	
	^term predicate! !

!AnnotatedTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: anIndex
	"	^term predicateInEnv: anEnv startAt: anIndex	"

	
	^term predicateInEnv: anEnv startAt: anIndex! !

!AnnotatedTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:02'!
prettyPrintOn: aStream scope: id
	
	term prettyPrintOn: aStream scope: id.
	aStream
		withAttributes:
			(Array
				with: TextColor red
				with: TextEmphasis bold) do:[
	aStream write: ' : '].
	annotation prettyPrintOn: aStream scope: id! !

!AnnotatedTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	
	aStream write: self class name , ' term: ('.
	term printForCompileOn: aStream.
	aStream write: ') annotation: ('.
	annotation printForCompileOn: aStream.
	aStream write: ')'! !

!AnnotatedTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	term printOn: aStream.
	aStream write: ' : '.
	annotation printOn: aStream! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	
	self definitionRepository: aRepository.
	term recursiveDefinitionRepository: aRepository.
	annotation recursiveDefinitionRepository: aRepository! !

!AnnotatedTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: env startAt: anIndex
	
	^term resolveIn: env startAt: anIndex! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
term
	
	^term! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
term: anObject
	
	term := anObject! !

!AnnotatedTerm methodsFor: 'initialize-release' stamp: 'sp 24/10/11 15:26'!
term: aTerm annotation: anAnnotationTerm
	
	term := aTerm.
	annotation := anAnnotationTerm! !

!AnnotatedTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	| newterm newAnnotation |
	newAnnotation := annotation transitiveLookupIn: anEnv startAt: anIndex.
	newterm := term transitiveLookupIn: anEnv startAt: anIndex.
	^(self class term: newterm annotation: newAnnotation)
		definitionRepository: self definitionRepository! !

!AnnotatedTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithAnnotatedTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!AnnotatedTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithAnnotatedTerm: aTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^(self term
		unifyWith: aTerm term
		inEnv: anEnv
		myIndex: sourceIndex
		hisIndex: targetIndex
		inSource: inSource)
		and:
			[self annotation
				unifyWith: aTerm annotation
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource]! !

!AnnotatedTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithAnnotatedTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!AnnotatedTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithAnnotatedTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!AnnotatedTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	
	^(term usesPredicate: aPredicate multiplicity: aNumber)
		or: [annotation usesPredicate: aPredicate multiplicity: aNumber]! !

ResolvableTerm subclass: #CallTerm
	instanceVariableNames: 'variable normalterm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!CallTerm commentStamp: '<historical>' prior: 0!
This class represent the CallTerm logic term. Call terms are logic terms that are called inside another repository than the one where they are defined. The syntax is:

?repvar->callterm(anything)!

!CallTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
variable: aVariable term: aTerm
	
	^self new variable: aVariable term: aTerm! !

!CallTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor callTermVisit: self! !

!CallTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asNormalTerm
	"Convert the callterm to a normalterm simply by returning its normalterm inside"

	
	^normalterm! !

!CallTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
callRepositoryInEnv: anEnv startAt: anIndex
	
	| localValue |
	localValue := variable soulDownIn: anEnv startAt: anIndex.
	^localValue = variable
		ifTrue: [self definitionRepository lookupBindingFor: variable]
		ifFalse: [localValue]! !

!CallTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
clauseNotFoundIn: repository inEnv: env startAt: anIndex
	
	(self
		dynamicPredicate: (self predicateInEnv: env startAt: anIndex)
		inEnv: env
		startAt: anIndex)
		ifTrue:
			[^SoulFactory current makeNativeResolver
				onBlock: [:res | res]
				withArguments: #(nil)]
		ifFalse:
			[self
				error:
					'Cannot resolve ' , self normalterm asString , ' in repository '
						, repository name]! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
definitionRepository: aRepository
	"See comment in AbstractTerm"

	
	definitionRepository := aRepository.
	normalterm definitionRepository: aRepository! !

!CallTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
dynamicPredicate: aPredicate inEnv: anEnv startAt: anIndex
	
	| evaluator result resRep |
	resRep := self callRepositoryInEnv: anEnv startAt: anIndex.
	(resRep addressForPredicate: 'dynamic')
		ifNil: [^false]
		ifNotNil:
			[evaluator := SoulFactory current evaluatorForLogicQuery
				eval: 'if dynamic(' , aPredicate , ')'
				in: self definitionRepository.
			result := evaluator nextResult.
			^result success]! !

!CallTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCallTerm
	
	^true! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^normalterm multiplicity! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
normalterm
	
	^normalterm! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
predicate
	
	| normalPredicate |
	normalPredicate := normalterm predicate.
	^normalPredicate == nil
		ifTrue: [nil]
		ifFalse: ['Q' , variable name , normalterm predicate]! !

!CallTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: anIndex
	
	^self normalterm predicateInEnv: anEnv startAt: anIndex! !

!CallTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' variable: ('.
	variable printForCompileOn: aStream.
	aStream write: ') term: ('.
	normalterm printForCompileOn: aStream.
	aStream write: ')'! !

!CallTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	variable printOn: aStream.
	aStream write: '->'.
	normalterm printOn: aStream! !

!CallTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: anEnv startAt: anIndex
	"resolve as a normal clause: do lookup in repository and create a  
	resolver on it"

	
	| foundClauses resRep |
	resRep := self callRepositoryInEnv: anEnv startAt: anIndex.
	resRep compile.
	normalterm recursiveDefinitionRepository: resRep.
	foundClauses := resRep
		normalAddressLookup:
			(normalterm
				lexicalAddressInEnv: anEnv
				startAt: anIndex
				repository: resRep
				ifAbsent:
					[^self clauseNotFoundIn: resRep inEnv: anEnv startAt: anIndex])
		multiplicity: self multiplicity.
	^Resolver fromClause: normalterm toClauses: foundClauses inEnv: anEnv
		startAt: anIndex! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	"See comment in AbstractTerm"

	
	^normalterm usesPredicate: aPredicate multiplicity: aNumber! !

!CallTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
variable
	
	^variable! !

!CallTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
variable: aVariable term: aTerm
	"A callterm contains a normalterm and a repository variable"

	
	variable := aVariable.
	normalterm := aTerm! !

ResolvableTerm subclass: #CompoundTerm
	instanceVariableNames: 'functor termsequence astlink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!CompoundTerm commentStamp: '<historical>' prior: 0!
This class represents logic formulas. That are logic terms having a functor (like a name) and a list of arguments.!

!CompoundTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
functor: aFunctor termSequence: aTermSequence
	
	^self new setFunctor: aFunctor setTermSequence: aTermSequence! !

!CompoundTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
functor: aFunctor termSequence: aTermSequence astLink: aVariable
	
	^self new initialize setFunctor: aFunctor setTermSequence: aTermSequence
		astLink: aVariable! !

!CompoundTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor compoundVisit: self! !

!CompoundTerm methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addTermFirst: term
	
	termsequence addTermFirst: term! !

!CompoundTerm methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConclusionWithSize: aSize
	
	termsequence terms add: (SoulFactory current makeVariable name: 'tokens1').
	termsequence terms
		add: (SoulFactory current makeVariable name: 'tokens' , aSize asString).
	^self! !

!CompoundTerm methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConditionFor: counter
	
	(#(#not #and #or) includes: self functor name)
		ifTrue: [self error: 'Cannot use and/or/not in DCG production']
		ifFalse:
			[termsequence terms
				add: (SoulFactory current makeVariable name: 'tokens' , counter asString).
			termsequence terms
				add:
					(SoulFactory current makeVariable name: 'tokens' , (counter + 1) asString)]! !

!CompoundTerm methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asFact
	
	^SoulFactory current makeFact conclusion: self! !

!CompoundTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asQuery
	
	^Query condition: (TermSequence terms: (OrderedCollection with: self))! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
asSourceString
	
	^self asSourceStringIn: nil startAt: nil! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
astLink
	
	^astlink! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
astLink: anASTLink
	
	astlink := anASTLink! !

!CompoundTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
clauseNotFoundIn: repository inEnv: env startAt: anIndex
	
	| val |
	((self functor isKindOf: Variable)
		and:
			[(val := self functor soulDownIn: env startAt: anIndex) class ==
					BlockClosure])
		ifTrue: [^self resolveAsApplication: val inEnv: env startAt: anIndex].
	^super clauseNotFoundIn: repository inEnv: env startAt: anIndex! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
functor
	"return our functor"

	
	^functor! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
functor: newFunctor
	
	functor := newFunctor! !

!CompoundTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
functorMatchesMultiplicity
	
	^true! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
isASTLinkedIn: anEnv startAt: index
	
	^((self astLink soulDownIn: anEnv startAt: index) isKindOf: Variable) not! !

!CompoundTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCompoundTerm
	
	^true! !

!CompoundTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isIndexable
	
	^false! !

!CompoundTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToCompoundTerm: self! !

!CompoundTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToCompoundTerm: aCompoundTerm
	
	^self termSequence isSLGEquivalentTo: aCompoundTerm termSequence! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"this method is called if we are printed inside a list"

	
	aStream write: ','.
	self printOn: aStream! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
multiplicity
	"return our multiplicity: the number of arguments of the compound"

	
	^self termSequence size! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
predicate
	"See comment in AbstractTerm"

	
	^self functor isIndexable
		ifTrue: [self functor asPredicateNameInEnv: nil startAt: nil]
		ifFalse: [nil]! !

!CompoundTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: anIndex
	
	^self functor asPredicateNameInEnv: anEnv startAt: anIndex! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	
	| newId |
	id = #asClause
		ifTrue: [newId := #asClause]
		ifFalse: [newId := #asTerm].
	functor prettyPrintAsFunctorOn: aStream scope: newId.
	aStream write: '('.
	termsequence
		do: [:aterm | aterm prettyPrintOn: aStream scope: #asTerm]
		separatedBy: [aStream write: ','].
	aStream write: ')'! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	| newId |
	id = #asClause
		ifTrue: [newId := #asClause]
		ifFalse: [newId := #asTerm].
	functor printAsPrologFunctorOn: aStream scope: newId in: env startAt: idx.
	aStream write: '('.
	termsequence
		do:
			[:aterm | aterm printAsPrologOn: aStream scope: #asTerm in: env startAt: idx]
		separatedBy: [aStream write: ','].
	aStream write: ')'! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' functor:('.
	functor printForCompileOn: aStream.
	aStream write: ') termSequence:('.
	termsequence printForCompileOn: aStream.
	aStream write: ')'! !

!CompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	functor printAsFunctorOn: aStream.
	aStream write: '('.
	termsequence
		do: [:aterm | aterm printOn: aStream]
		separatedBy: [aStream write: ','].
	aStream write: ')'! !

!CompoundTerm methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	"see comment in AbstractTerm"

	
	self definitionRepository: aRepository.
	termsequence recursiveDefinitionRepository: aRepository! !

!CompoundTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveAsApplication: aBlockClosure inEnv: env startAt: anIndex
	"I resolve a CompoundTerm whose functor is a BlockClosure as the application of the BlockClosure to the CompoundTerm's arguments."

	
	| numArgs downedTermArgs |
	downedTermArgs := self termSequence terms asArray
		collect: [:term | term soulDownIn: env startAt: anIndex].
	numArgs := aBlockClosure numArgs.
	numArgs = self multiplicity
		ifTrue:
			[^(aBlockClosure valueWithArguments: downedTermArgs) soulUp resolveIn: env
				startAt: anIndex].
	numArgs = (self multiplicity - 1)
		ifTrue:
			[^SoulFactory current makeNativeResolver
				onBlock:
					[:arg | 
					(downedTermArgs last
						unifyWith: arg soulUp
						inEnv: env
						myIndex: anIndex
						hisIndex: (env startUnify: 0)
						inSource: false)
						ifFalse:
							[env rollback.
							nil]
						ifTrue:
							[SoulFactory current makeResolutionResult
								condition: SoulFactory current makeTermSequence new
								rollbackInfo: env rollbackInfo
								conditionsEnvIndex: nil
								cutState: false]]
				withArguments:
					(OrderedCollection
						with:
							(aBlockClosure valueWithArguments: (downedTermArgs allButLast: 1)))].
	^SoulFactory current makeNativeResolver
		onBlock: [:res | res]
		withArguments: #(nil)! !

!CompoundTerm methodsFor: 'initialize' stamp: 'AndyKellens 11/2/2011 09:53'!
setFunctor: aFunctor setTermSequence: aTermSequence
	"initialization of the compound"

	
	functor := aFunctor.
	termsequence := aTermSequence.
	astlink := HiddenVariable name: SoulFactory uniqueName! !

!CompoundTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setFunctor: aFunctor setTermSequence: aTermSequence astLink: aVariable
	"initialization of the compound"

	
	functor := aFunctor.
	termsequence := aTermSequence.
	astlink := aVariable! !

!CompoundTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"downing a compoundterm means downing the functor and downing the termsequence. HOWEVER: we must up the functor again because it gets included 	in the compound again"

	
	| newTermSequence value |
	value := astlink soulDownIn: anEnv startAt: anIndex.	"Need for a cleaner solution here... this requires me to work a bit on the environment too though"

	(value isKindOf: Variable)
		ifTrue:
			[newTermSequence := termsequence soulDownIn: anEnv startAt: anIndex.
			^(self class
				functor: (functor soulDownIn: anEnv startAt: anIndex) soulUp
				termSequence: newTermSequence
				astLink: value) definitionRepository: self definitionRepository]
		ifFalse: [^value]! !

!CompoundTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulUp
	"upping a compoundterm means returning myself because all my  
	subterms are automatically already upped"

	"| newSubTerms | 
	newSubTerms _ terms 
	collect: [:subTerm | subTerm qsoulUp]. 
	^ self class functor: functor qsoulUp terms: newSubTerms"

	
	^self! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
termSequence
	"return the termsequence, containing our subterms"

	
	^termsequence! !

!CompoundTerm methodsFor: 'access' stamp: 'ReinoutStevens 11/15/2011 11:38'!
terms
	"return the termsequence, containing our subterms"

	
	^termsequence terms! !

!CompoundTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	"See comment in AbstractTerm"

	
	| newTermSequence |
	newTermSequence := termsequence transitiveLookupIn: anEnv startAt: anIndex.
	^(self class
		functor: (functor transitiveLookupIn: anEnv startAt: anIndex)
		termSequence: newTermSequence
		astLink: (astlink transitiveLookupIn: anEnv startAt: anIndex))
		definitionRepository: self definitionRepository! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"unify a compound with a compound => if the arities are the  
	same and the functors unify, then recursively unify the subterms"

	
	| otherTermSequence myAst hisAst |
	(((myAst := astlink soulDownIn: anEnv startAt: myIndex) isKindOf: Variable)
		not
		and:
			[((hisAst := aCompound astLink soulDownIn: anEnv startAt: hisIndex)
				isKindOf: Variable) not])
		ifTrue:
			[^myAst unifyWith: hisAst inEnv: anEnv myIndex: myIndex hisIndex: hisIndex
				inSource: inSource]
		ifFalse:
			[self multiplicity = aCompound multiplicity
				&
					(self functor
						unifyWith: aCompound functor
						inEnv: anEnv
						myIndex: myIndex
						hisIndex: hisIndex
						inSource: inSource)
				ifTrue:
					[(astlink
						unifyWith: aCompound astLink
						inEnv: anEnv
						myIndex: myIndex
						hisIndex: hisIndex
						inSource: inSource) ifFalse: [^false].
					otherTermSequence := aCompound termSequence.
					termsequence terms
						with: otherTermSequence terms
						do:
							[:myTerm :otherTerm | 
							(myTerm unifyWith: otherTerm inEnv: anEnv myIndex: myIndex hisIndex:
									hisIndex inSource: inSource) ifFalse: [^false]].
					^true]
				ifFalse: [^false]]! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"a compound unifies with a quotedCode"

	
	^aQCTerm
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithResultMessageTerm: aRM inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"ResultMessageTerm implements this"

	
	^aRM
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: inSource not! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: stterm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"this is handled in SmalltalkTerm itself"

	
	^stterm
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aVarArgsCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aVarArgsCompound
		unifyWithCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!CompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	| myAst |
	^(myAst := astlink soulDownIn: anEnv startAt: myIndex) class ==
			UppedASTObject
		ifTrue:
			[self halt.
			aVariable
				unifyWithUppedASTObject: myAst
				inEnv: anEnv
				myIndex: hisIndex
				hisIndex: myIndex
				inSource: inSource not]
		ifFalse:
			[aVariable
				unifyWithCompoundTerm: self
				inEnv: anEnv
				myIndex: hisIndex
				hisIndex: myIndex
				inSource: inSource not]! !

!CompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	"This method is used by repositories to look up all predicated used inside a clause."

	
	(self functor isKindOf: Variable)
		ifFalse:
			[self predicate = aPredicate & (self multiplicity = aNumber)
				ifTrue: [^true]].
	^self termSequence usesPredicate: aPredicate multiplicity: aNumber! !

CompoundTerm subclass: #ListTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!ListTerm commentStamp: '<historical>' prior: 0!
Represent a list term. They are simulated through compound terms, which is why it's a subclass of it.

!

!ListTerm class methodsFor: 'constants' stamp: 'AndyKellens 11/2/2011 09:53'!
ListTermFunctor
	"What kind of functor will we use to simulate lists as compound terms? "

	
	^SoulFactory current makeConstant name: #'.'! !

!ListTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
newFromCollection: aCollection
	
	^self new
		addChildrenInitial:
			(OrderedCollection
				with: (TermSequence terms: (aCollection collect: [:e | e soulUp])))! !

!ListTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
termSequence: aTermSequence
	
	^self new addChildrenInitial: (OrderedCollection with: aTermSequence)! !

!ListTerm methodsFor: 'constants' stamp: 'AndyKellens 11/2/2011 09:53'!
ListTermFunctor
	"What kind of functor will we use to simulate lists as compound terms?"

	
	^SoulFactory current makeConstant name: #'.'! !

!ListTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor listTermVisit: self! !

!ListTerm methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addChildrenInitial: aCollection
	"this method is called by the parser to construct a parsetree"

	"lists are simulated by compounds containing the head of the list as one 
	subterm and a listterm (compound) containing the rest of the list"

	"aCollection is a Collection that contains a TermSequence"

	
	| holdFirst holdSecond |
	holdSecond := EmptyListConstant new.
	aCollection first
		reverseDo:
			[:X | 
			| holdCollection |
			holdFirst := X.
			holdCollection := OrderedCollection with: holdFirst with: holdSecond.
			holdSecond := self makeListCompound: holdCollection].
	aCollection first size = 0
		ifTrue: [^holdSecond]
		ifFalse:
			[^self
				setFunctor: holdSecond functor
				setTermSequence: holdSecond termSequence]! !

!ListTerm methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConditionFor: counter
	
	| tokens1var tokens2var pairterm terminals |
	terminals := OrderedCollection new.
	self listDo: [:aTerm | terminals add: aTerm].
	tokens1var := SoulFactory current makeVariable
		name: 'tokens' , counter asString.
	tokens2var := SoulFactory current makeVariable
		name: 'tokens' , (counter + 1) asString.
	pairterm := SoulFactory current makePair new
		addChildrenInitial:
			(OrderedCollection
				with: (TermSequence terms: terminals)
				with: tokens2var).
	^SoulFactory current makeCompound
		functor: (Constant name: #equals)
		termSequence:
			(TermSequence terms: (OrderedCollection with: tokens1var with: pairterm))! !

!ListTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
astLink: anASTLink
	
	astlink := anASTLink! !

!ListTerm methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
collectIn: anEnv startAt: anIndex
	
	| newCollection |
	newCollection := OrderedCollection new.
	self
		listDo:
			[:elt | newCollection add: (elt soulDownIn: anEnv startAt: anIndex)].
	^newCollection! !

!ListTerm methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
listDo: aBlock
	
	aBlock value: termsequence terms first.
	termsequence terms last listDo: aBlock! !

!ListTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"this method is called when we are printed inside another list"

	
	aStream write: ','.
	termsequence terms first printOn: aStream.
	(termsequence terms last isKindOf: ListTerm)
		| (termsequence terms last isKindOf: EmptyListConstant)
		ifTrue: [termsequence terms last listPrintOn: aStream]
		ifFalse:
			[aStream write: '|'.
			termsequence terms last printOn: aStream]! !

!ListTerm methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
makeListCompound: aCollection
	"this is a help method for the parsetree construction"

	
	^self class
		functor: self class ListTermFunctor
		termSequence: (TermSequence terms: aCollection)! !

!ListTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyListPrintOn: aStream scope: id
	"this method is called when we are printed inside another list"

	
	aStream write: ','.
	termsequence terms first prettyPrintOn: aStream scope: id.
	(termsequence terms last isKindOf: ListTerm)
		| (termsequence terms last isKindOf: EmptyListConstant)
		ifTrue: [termsequence terms last prettyListPrintOn: aStream scope: id]
		ifFalse:
			[aStream write: '|'.
			termsequence terms last prettyPrintOn: aStream scope: id]! !

!ListTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"

	
	aStream write: '<'.
	termsequence terms first prettyPrintOn: aStream scope: id.
	(termsequence terms last isKindOf: ListTerm)
		| (termsequence terms last isKindOf: EmptyListConstant)
		ifTrue: [termsequence terms last prettyListPrintOn: aStream scope: id]
		ifFalse:
			[aStream write: '|'.
			termsequence terms last prettyPrintOn: aStream scope: id].
	aStream write: '>'! !

!ListTerm methodsFor: 'printing' stamp: 'AndyKellens 11/2/2011 09:53'!
printAsPrologListOn: aStream scope: id in: env startAt: idx
	"this method is called when we are printed inside another list"

	
	aStream write: ','.
	termsequence terms first printAsPrologOn: aStream scope: id in: env
		startAt: idx.
	termsequence terms last species == SoulFactory current makeVariable
		ifTrue:
			[aStream write: '|'.
			termsequence terms last printAsPrologOn: aStream scope: id in: env
				startAt: idx]
		ifFalse:
			[termsequence terms last printAsPrologListOn: aStream scope: id in: env
				startAt: idx]! !

!ListTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: scope in: env startAt: idx
	"see comment in AbstractTerm"

	
	aStream write: '['.
	termsequence terms first printAsPrologOn: aStream scope: #clause in: env
		startAt: idx.
	termsequence terms last printAsPrologListOn: aStream scope: #clause in:
			nil startAt: nil.
	aStream write: ']'! !

!ListTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: '<'.
	termsequence terms first printOn: aStream.
	(termsequence terms last isKindOf: ListTerm)
		| (termsequence terms last isKindOf: EmptyListConstant)
		ifTrue: [termsequence terms last listPrintOn: aStream]
		ifFalse:
			[aStream write: '|'.
			termsequence terms last printOn: aStream].
	aStream write: '>'! !

!ListTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: anEnv startAt: anIndex
	"See comment in AbstractTerm"

	
	self error: 'Cannot resolve a ' , self class asString! !

!ListTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setFunctor: aFunctor setTermSequence: aTermSequence
	"initialization of the compound"

	
	functor := aFunctor.
	termsequence := aTermSequence.
	astlink := nil! !

!ListTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"downing a compoundterm means downing myself and downing my  
	subterms. HOWEVER: We must up the subterms because they are  
	included in a listterm"

	
	| newTermSequence |
	newTermSequence := termsequence soulDownIn: anEnv startAt: anIndex.
	^self class functor: functor termSequence: newTermSequence! !

!ListTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	| newTermSequence |
	newTermSequence := termsequence transitiveLookupIn: anEnv startAt: anIndex.
	^self class functor: functor termSequence: newTermSequence! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"unify a compound with a compound => if the arities are the  
	same and the functors unify, then recursively unify the subterms"

	
	| otherTermSequence |
	self multiplicity = aCompound multiplicity
		&
			(self functor
				unifyWith: aCompound functor
				inEnv: anEnv
				myIndex: myIndex
				hisIndex: hisIndex
				inSource: inSource)
		ifTrue:
			[otherTermSequence := aCompound termSequence.
			termsequence terms
				with: otherTermSequence terms
				do:
					[:myTerm :otherTerm | 
					(myTerm unifyWith: otherTerm inEnv: anEnv myIndex: myIndex hisIndex:
							hisIndex inSource: inSource) ifFalse: [^false]].
			^true]
		ifFalse: [^false]! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self unifyWithCompoundTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex:
			hisIndex inSource: inSource! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"a compound unifies with a quotedCode"

	
	^aQCTerm
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithResultMessageTerm: aRM inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"ResultMessageTerm implements this"

	
	^aRM
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: inSource not! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: stterm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"this is handled in SmalltalkTerm itself"

	
	^stterm
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aVarArgsCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aVarArgsCompound
		unifyWithListTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!ListTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	| myAst |
	^(myAst := astlink soulDownIn: anEnv startAt: myIndex) class ==
			UppedASTObject
		ifTrue:
			[self halt.
			aVariable
				unifyWithUppedASTObject: myAst
				inEnv: anEnv
				myIndex: hisIndex
				hisIndex: myIndex
				inSource: inSource not]
		ifFalse:
			[aVariable
				unifyWithListTerm: self
				inEnv: anEnv
				myIndex: hisIndex
				hisIndex: myIndex
				inSource: inSource not]! !

ListTerm subclass: #PairTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!PairTerm commentStamp: '<historical>' prior: 0!
collect
	
	|newCollection|
	newCollection := OrderedCollection new.
	self listDo: [:elt | newCollection add: elt ].
	^newCollection!

!PairTerm methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addChildrenInitial: aCollection
	"this is called by the parser to construct a parsetree"

	"aCollection is een Collection die een QSOULTermSequence bevat"

	
	| holdFirst holdSecond |
	holdSecond := aCollection last.
	aCollection first
		reverseDo:
			[:X | 
			| holdCollection |
			holdFirst := X.
			holdCollection := OrderedCollection with: holdFirst with: holdSecond.
			holdSecond := self makeListCompound: holdCollection].
	^self
		setFunctor: holdSecond functor
		setTermSequence: holdSecond termSequence! !

!PairTerm methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
collectIn: anEnv startAt: anIndex
	
	| newCollection |
	newCollection := OrderedCollection new.
	newCollection
		add: (termsequence terms first soulDownIn: anEnv startAt: anIndex).
	newCollection
		addAll:
			((termsequence terms last soulDownIn: anEnv startAt: anIndex) collectIn:
					anEnv startAt: anIndex).
	^newCollection! !

!PairTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	"see comment in AbstractTerm"

	
	aStream write: '['.
	termsequence terms first printAsPrologOn: aStream scope: id in: env
		startAt: idx.
	termsequence terms last isVariable
		ifTrue:
			[aStream write: '|'.
			termsequence terms last printAsPrologOn: aStream scope: id in: nil
				startAt: nil]
		ifFalse:
			[termsequence terms last printAsPrologListOn: aStream scope: id in: nil
				startAt: nil].
	aStream write: ']'! !

CompoundTerm subclass: #MessageTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!MessageTerm methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addKeyword: keyword term: term
	
	functor addKeyword: keyword.
	termsequence addTermFirst: term! !

!MessageTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
arguments
	
	^termsequence terms allButFirst: 1! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
functorMatchesMultiplicity
	
	^functor multiplicity = self multiplicity! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isKeyworded
	
	^functor isKindOf: KeywordFunctor! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isKeywordedMessage
	
	^functor isKindOf: MessageFunctor! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isMessage
	
	^self isKeywordedMessage or: [self isUnaryMessage]! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isMultiPart
	
	^self isKeywordedMessage or: [self isKeyworded]! !

!MessageTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUnaryMessage
	
	^functor isKindOf: UnaryMessageFunctor! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintAsKeywordArgOn: aStream scope: id
	"UGLY: should change type test to double dispatching"

	
	self isMultiPart ifTrue: [aStream write: '('].
	self prettyPrintOn: aStream scope: id.
	self isMultiPart ifTrue: [aStream write: ')']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintKeywordMessageStyleOn: aStream scope: id
	
	| newId |
	id = #asClause
		ifTrue: [newId := #asClause]
		ifFalse: [newId := #asTerm].
	self receiver prettyPrintAsKeywordArgOn: aStream scope: newId.
	aStream write: ' '.
	self functor keywords
		with: (termsequence terms allButFirst: 1)
		do:
			[:keyword :aterm | 
			keyword prettyPrintAsKeywordOn: aStream scope: newId.
			aStream write: ': '.
			aterm prettyPrintAsKeywordArgOn: aStream scope: #asTerm]
		separatedBy: [aStream write: ' ']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintKeywordStyleOn: aStream scope: id
	
	| newId |
	id = #asClause
		ifTrue: [newId := #asClause]
		ifFalse: [newId := #asTerm].
	self functor keywords
		with: termsequence terms
		do:
			[:keyword :aterm | 
			keyword prettyPrintAsKeywordOn: aStream scope: newId.
			aStream write: ': '.
			aterm prettyPrintAsKeywordArgOn: aStream scope: newId]
		separatedBy: [aStream write: ' ']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	"UGLY: should really be replaced with nice double dispatching"

	
	self isKeyworded
		ifTrue: [^self prettyPrintKeywordStyleOn: aStream scope: id].
	self isKeywordedMessage
		ifTrue: [^self prettyPrintKeywordMessageStyleOn: aStream scope: id].
	self isUnaryMessage
		ifTrue: [^self prettyPrintUnaryMessageStyleOn: aStream scope: id].
	self halt: 'terribly wrong'! !

!MessageTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:08'!
prettyPrintUnaryMessageStyleOn: aStream scope: id
	
	termsequence terms first prettyPrintOn: aStream scope: #asTerm.
	aStream write: ' '.
	aStream withAttributes:
			(Array
				with: TextColor black
				with: TextEmphasis bold) do:[
	functor message prettyPrintOn: aStream scope: #asTerm.]! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsKeywordArgOn: aStream
	"UGLY: should change type test to double dispatching"

	
	self isMultiPart ifTrue: [aStream write: '('].
	self printOn: aStream.
	self isMultiPart ifTrue: [aStream write: ')']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printKeywordMessageStyleOn: aStream
	"see comment in AbstractTerm"

	
	self receiver printAsKeywordArgOn: aStream.
	aStream write: ' '.
	self functor keywords
		with: (termsequence terms allButFirst: 1)
		do:
			[:keyword :aterm | 
			keyword printOn: aStream.
			aStream write: ': '.
			aterm printAsKeywordArgOn: aStream]
		separatedBy: [aStream write: ' ']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printKeywordStyleOn: aStream
	"see comment in AbstractTerm"

	
	self functor keywords
		with: termsequence terms
		do:
			[:keyword :aterm | 
			keyword printOn: aStream.
			aStream write: ': '.
			aterm printAsKeywordArgOn: aStream]
		separatedBy: [aStream write: ' ']! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"UGLY: should really be replaced with nice double dispatching"

	"No... really?"

	
	self isKeyworded ifTrue: [^self printKeywordStyleOn: aStream].
	self isKeywordedMessage
		ifTrue: [^self printKeywordMessageStyleOn: aStream].
	self isUnaryMessage ifTrue: [^self printUnaryMessageStyleOn: aStream].
	self halt: 'terribly wrong'! !

!MessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printUnaryMessageStyleOn: aStream
	
	termsequence terms first printOn: aStream.
	aStream write: ' '.
	functor message printOn: aStream! !

!MessageTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
receiver
	"Only makes sense when the functor of this compound is a message-like functor"

	
	^termsequence terms at: 1! !

!MessageTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
selector
	
	^functor name! !

MessageTerm subclass: #RegexTerm
	instanceVariableNames: 'goals generatedGoals identifier initialGoal endGoal failGoal generatedClauses edgeTerm startWorld endWorld variablesVar resultVar pathVar versionTrans'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

RegexTerm class
	instanceVariableNames: 'nrGeneratedGoals keywords nrRegexTerms generatedClauses'!

RegexTerm class
	instanceVariableNames: 'nrGeneratedGoals keywords nrRegexTerms generatedClauses'!

!RegexTerm class methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
addRegexTerm
	
	nrRegexTerms := self nrRegexTerms + 1.
	^nrRegexTerms! !

!RegexTerm class methodsFor: 'clause generation' stamp: 'ReinoutStevens 1/16/2012 13:33'!
generateRule: aString definitionRepository: aRepository
	
	| newRule |
	newRule := (SoulFactory current makeParser
		parse: aString
		onError: [:a :b | self error: 'parse error in generated regexrule'])
		clauses first.
	generatedClauses add: newRule.
	^newRule! !

!RegexTerm class methodsFor: 'clause generation' stamp: 'sp 24/10/11 15:26'!
generatedClauses
	
	^generatedClauses! !

!RegexTerm class methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
incNrGeneratedGoals
	
	nrGeneratedGoals := nrGeneratedGoals + 1! !

!RegexTerm class methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
keywords
	
	keywords
		ifNil:
			[keywords := Dictionary new.
			keywords at: #manyOf put: #processManyOf:remainingConditions:.
			keywords at: #or put: #processOr:remainingConditions:.	"keywords at: #not put: #processNot:remainingConditions: ."
	"keywords at: #and put: #processAnd:remainingConditions: ."

			keywords at: #notSoManyOf put: #processLazyManyOf:remainingConditions:.
			keywords at: #plus put: #processPlus].
	^keywords! !

!RegexTerm class methodsFor: 'instance creation' stamp: 'ReinoutStevens 1/12/2012 15:29'!
newFromMessageTerm: aMessageTerm withGoals: goals
	
	| term |
	term := RegexTerm
		functor: aMessageTerm functor
		termSequence: aMessageTerm termSequence
		astLink: aMessageTerm astLink.
	term validateRegexTerms.
	term goals: goals.
	^term! !

!RegexTerm class methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
nrGeneratedGoals
	
	nrGeneratedGoals ifNil: [nrGeneratedGoals := 0].
	^nrGeneratedGoals! !

!RegexTerm class methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
nrRegexTerms
	
	nrRegexTerms ifNil: [nrRegexTerms := 0].
	^nrRegexTerms! !

!RegexTerm class methodsFor: 'clause generation' stamp: 'sp 24/10/11 15:26'!
resetGeneratedClauses
	
	generatedClauses := OrderedCollection new! !

!RegexTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
additionalClausesForDefinitionRepository
	
	^generatedClauses! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
allAfter: anObject in: aCollection
	
	| startIndex |
	startIndex := aCollection indexOf: anObject.
	^aCollection
		copyFrom: startIndex + 1
		to: aCollection size! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
allBefore: anObject in: aCollection
	
	^aCollection copyUpTo: anObject! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
connect: from to: to class: aRegexTransitionClass
	
	^from connectTo: to transition: aRegexTransitionClass! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
detectRegexPrimitive: aTermCollection
	
	| termFunctor |
	aTermCollection
		do:
			[:term | 
			term isCompoundTerm
				ifTrue:
					[termFunctor := term functor name.
					(RegexTerm keywords
						at: termFunctor
						ifAbsent: [nil]) ifNotNil: [^term]]].
	^nil! !

!RegexTerm methodsFor: 'converting' stamp: 'ReinoutStevens 1/16/2012 13:35'!
fixTermMagic
	"in the hope of making our regexterm automagically work with the rest we are correcting our functor and termsequence,
	as SOUL will automagically run the correct query, generate a rainbow and some unicorns. If not all hell breaks lose
	and I will be a sad bunny"

	
	| collection startState endState path |
	collection := OrderedCollection with: (self getResultVariable).
	startState := self getStartWorld.
	endState := self getEndWorld.
	path := self getPathVariable.
	self prepareForModifyingTermsequence.
	
	collection add: startState.
	collection add: endState.
	collection add: path.
	termsequence := TermSequence terms: collection.
	self isExistential
		ifTrue:
			[functor := MessageFunctor
				name: 'somepath' , self identifier asString , ':endWorld:path:']
		ifFalse:
			[self isUniversal
				ifTrue:
					[functor := MessageFunctor
						name: 'allpaths' , self identifier asString , ':endWorld:']
				ifFalse:
					[self error:
							'regexterm without a universal (a) or existentional (e) quantifier']]! !

!RegexTerm methodsFor: 'converting' stamp: 'ReinoutStevens 1/12/2012 15:28'!
generateClauses
	
	RegexTerm resetGeneratedClauses.
	generatedGoals
		do:
			[:goal | goal generateTransitions: self term: (self getEdgeTerm) definitionRepository: nil].
	self isExistential ifTrue: [self generateExistential: nil].
	self isUniversal ifTrue: [self generateUniversal: nil].
	generatedClauses := RegexTerm generatedClauses! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 14:43'!
generateExistential: aDefinitionRepository
	
	| nproduct nreachBase nreachRecur somePaths failnpattern coll |
	nproduct := 'nproduct' , self identifier asString ,
			'(?phi, ?n1, ?a1, ?n2, ?a2, ?current, ?next, ?processedPath, ?differenceVariable, ?restPath, ?newDifferenceVariable, ?newRestPath) if ' , 
			'npattern' , self identifier asString ,
			'(?phi, ?a1, ?a2, ?current, ?next, ?n1, ?n2, ?processedPath, ?differenceVariable, ?restPath, ?newDifferenceVariable, ?newRestPath)'.
	failnpattern := 'npattern' , self identifier asString ,
			'(?phi, ?a1, ?a2, ?n1, ?n2, ?current, ?next, ?processedPath, ?differenceVariable, ?restPath, ?newDifferenceVariable, ?newRestPath) if !!, [ false ]'.
	nreachBase := 'nreach' , self identifier asString ,
			'(?phi, ?n1,?p1, ?n1, ?p1, ?current, ?next, ?processedPath, ?differenceVariable, <>) if [true]'.
	nreachRecur := 'nreach' , self identifier asString ,
			'(?phi, ?n1,?p1, ?n2, ?p2, ?current, ?next, ?processedPath, ?differenceVariable, ?restPath) if ' , 
			'nproduct' , self identifier asString ,
			'(?phi, ?n1, ?p1, ?na, ?pa, ?current, ?next, ?processedPath, ?differenceVariable, ?restPath, ?newDifferenceVariable, ?newRestPath),' , 
			'nreach' , self identifier asString , '(?phi, ?na, ?pa, ?n2, ?p2, ?next, ?neext, ?processedPath, ?newDifferenceVariable, ?newRestPath)'.
	somePaths := '?phi somepath' , self identifier asString ,
			': ?startWorld endWorld: ?endWorld path: ?path if ' , 
			'nreach', self identifier asString , 
			'(?phi, ?startWorld,' , initialGoal goalId asString , ' , ?endWorld,' , 
			endGoal goalId asString, ', ?startWorld, ?next,', 
			'?differenceVariable, ?differenceVariable, ?path)'.
	coll := OrderedCollection with: nproduct with: nreachBase with:
			nreachRecur with: somePaths.
	coll add: failnpattern.
	coll
		do:
			[:rule | RegexTerm generateRule: rule definitionRepository: aDefinitionRepository]! !

!RegexTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
generateFailGoal
	
	| sequence |
	failGoal
		ifNil:
			[sequence := (SoulFactory current makeParser
				parse: ' if [ false ]'
				onError: [:a :b | self error: 'parse error in generated regexrule'])
				condition terms.
			failGoal := self generateGoal: sequence].
	^failGoal! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/12/2012 15:33'!
generateGoal: aCollection
	
	| generated |
	generated := RegexGoal
		newWith: aCollection
		id: RegexTerm nrGeneratedGoals
		variables: (self getVariables).
	generatedGoals add: generated.
	RegexTerm incNrGeneratedGoals.
	^generated! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
generateGoals: aCollection
	"returns a collection existing out of collections with newly added goals"

	
	| primitive allBefore allAfter newBefores newAfters first last |
	aCollection isEmpty ifTrue: [^nil].
	primitive := self detectRegexPrimitive: aCollection.
	primitive isNil
		ifTrue:
			["no more primitives to process, so we just create a goal"

			^self processCollection: aCollection]
		ifFalse:
			[allBefore := self allBefore: primitive in: aCollection.
			allAfter := self allAfter: primitive in: aCollection.
			newBefores := nil.
			newAfters := nil.
			allBefore isEmpty
				ifFalse: [newBefores := self processCollection: allBefore].
			newAfters := self
				perform: (RegexTerm keywords at: primitive functor smalltalkValue)
				withArguments: (Array with: primitive with: allAfter).
			newBefores isNil
				ifFalse:
					[first := newBefores at: 1.
					last := newBefores at: 2.
					newAfters isNil
						ifFalse:
							[self
								connect: last
								to: (newAfters at: 1)
								class: RegexTransition]].
			newBefores isNil ifTrue: [first := newAfters at: 1].
			last := newAfters at: 2].
	^Array with: first with: last! !

!RegexTerm methodsFor: 'converting' stamp: 'ReinoutStevens 1/12/2012 15:28'!
generateStates
	
	| firstlast end |
	firstlast := self generateGoals: goals termSequence terms.
	initialGoal := firstlast at: 1.
	end := firstlast at: 2.
	endGoal := self generateGoal: OrderedCollection new.
	self connect: end to: endGoal class: RegexEndTransition.
	self fixTermMagic.
	self generateClauses! !

!RegexTerm methodsFor: 'private' stamp: 'AndyKellens 11/10/2011 13:11'!
generateUniversal: aDefinitionRepository
	
	| allPaths nproduct nreachBase nreachRecur failnpattern coll |
	self generateExistential: aDefinitionRepository.
	nproduct := 'nproductUniversal' , self identifier asString ,
			'(?phi, ?n1, ?a1, ?n2, ?a2) if ' , 'npatternUniversal'
		, self identifier asString , '(?phi, ?a1, ?a2, ?current, ?n1, ?n2)'.
	failnpattern := 'npatternUniversal' , self identifier asString ,
			'(?phi, ?a1, ?a2, ?current, ?n1, ?n2) if !!, [ false ]'.
	nreachBase := 'nreachUniversal' , self identifier asString ,
			'(?phi, ?n1,?p1, ?n1, ?p1) if [true]'.
	nreachRecur := 'nreachUniversal' , self identifier asString ,
			'(?phi, ?n1,?p1, ?n2, ?p2) if ' , 'nproductUniversal'
		, self identifier asString , '(?phi, ?n1, ?p1, ?na, ?pa),' ,
			'nreachUniversal' , self identifier asString ,
			'(?phi, ?na, ?pa, ?n2, ?p2)'.
	allPaths := '?phi allpaths' , self identifier asString ,
			': ?startWorld endWorld: ?endWorld if ' , '?phi somepath'
		, self identifier asString , ': ?startWorld endWorld: ?endWorld,' ,
			'nreachUniversal' , self identifier asString , '(?phi, ?startWorld,'
		, initialGoal goalId asString , ' , ?endWorld,' , endGoal goalId asString
		, ')'.
	coll := OrderedCollection with: nproduct with: nreachBase with:
			nreachRecur with: allPaths.
	coll add: failnpattern.
	coll
		do:
			[:rule | RegexTerm generateRule: rule definitionRepository: aDefinitionRepository]! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 12:50'!
getEdgeTerm
	"I N C E P T I O N"
	edgeTerm ifNil: [
		edgeTerm := self termSequence terms first termSequence terms first
	].
	^edgeTerm.! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 12:51'!
getEndWorld
	"I N C E P T I O N"
	endWorld ifNil: [
		endWorld := self termSequence terms first termSequence terms at: 3	
	].
	^endWorld.
	
	! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 12:54'!
getPathVariable 
	"I N C E P T I O N"
	pathVar ifNil: [ 
		pathVar := self termSequence terms first termSequence terms at: 5
	].
	^pathVar.! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 13:02'!
getResultVariable
	resultVar ifNil: [
		resultVar := self getVariables.		
	].
	^resultVar.
! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 13:34'!
getStartWorld
	"I N C E P T I O N"
	startWorld ifNil: [
		startWorld := self termSequence terms first termSequence terms at: 2.	
	].
	^startWorld! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 13:01'!
getVariables
	"I N C E P T I O N"
	variablesVar ifNil: [
		variablesVar :=	self termSequence terms first termSequence terms at: 4
	].
	^variablesVar.! !

!RegexTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
goals: anObject
	
	goals := anObject! !

!RegexTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
identifier
	
	^identifier! !

!RegexTerm methodsFor: 'initialize-release' stamp: 'sp 24/10/11 15:26'!
initialize
	
	super initialize.
	identifier := RegexTerm addRegexTerm.
	generatedGoals := OrderedCollection new! !

!RegexTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isExistential
	
	^goals functor name = #e! !

!RegexTerm methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUniversal
	
	^goals functor name = #a! !

!RegexTerm methodsFor: 'private' stamp: 'ReinoutStevens 1/16/2012 13:11'!
prepareForModifyingTermsequence
	"we save all the variables before modifying our own termsequence"
	self getEdgeTerm.
	self getEndWorld.
	self getPathVariable.
	self getResultVariable.
	self getStartWorld.
	self getVariables.! !

!RegexTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	
	self printOn: aStream! !

!RegexTerm methodsFor: 'printing' stamp: 'ReinoutStevens 1/12/2012 15:30'!
printOn: aStream
	goals printAsKeywordArgOn: aStream.
	aStream write: ' matches: graph('.
	(self getEdgeTerm) printOn: aStream.
	aStream write: ', '.
	(self getStartWorld) printOn: aStream.
	aStream write: ', '.
	(self getEndWorld) printOn: aStream.
	aStream write: ', '.
	(self getVariables) printOn: aStream.
	aStream write: ', '.
	(self getPathVariable) printOn: aStream.
	aStream write: ')'! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processCollection: aCollection
	
	| first prev curr |
	prev := self generateGoal: (OrderedCollection with: aCollection first).
	first := prev.
	(aCollection allButFirst: 1)
		do:
			[:term | 
			curr := self generateGoal: (OrderedCollection with: term).
			self connect: prev to: curr class: RegexTransition.
			prev := curr].
	^Array with: first with: prev! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processLazyManyOf: manyTerm remainingConditions: aCollection
	
	| firstLast first last regexLast remainingGoals |
	firstLast := self generateGoals: manyTerm termSequence terms.
	firstLast ifNil: [self error: 'manyOf with empty arguments'].
	first := firstLast at: 1.
	last := firstLast at: 2.
	regexLast := last.
	remainingGoals := self generateGoals: aCollection.
	remainingGoals isNil
		ifFalse:
			[self
				connect: first
				to: (remainingGoals at: 1)
				class: RegexEpsilonTransition.
			last := remainingGoals at: 2]
		ifTrue: [last := first].
	self connect: regexLast to: first class: RegexTransition.
	^Array with: first with: last! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processManyOf: manyTerm remainingConditions: aCollection
	
	| firstLast first last remainingGoals |
	firstLast := self generateGoals: manyTerm termSequence terms.
	firstLast ifNil: [self error: 'manyOf with empty arguments'].
	first := firstLast at: 1.
	last := firstLast at: 2.
	self connect: last to: first class: RegexTransition.
	remainingGoals := self generateGoals: aCollection.
	remainingGoals isNil
		ifFalse:
			[self
				connect: first
				to: (remainingGoals at: 1)
				class: RegexEpsilonTransition.
			last := remainingGoals at: 2]
		ifTrue: [last := first].
	^Array with: first with: last! !

!RegexTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
processNot: notTerm remainingConditions: aCollection
	
	| end notGoal newGoals remainingGoals fail |
	newGoals := self generateGoal: notTerm termSequence terms.
	newGoals ifNil: [self error: 'not term without any goals'].
	notGoal := self generateGoal: OrderedCollection new.
	self
		connect: notGoal
		to: (newGoals at: 1)
		class: RegexEpsilonTransition.
	remainingGoals := self generateGoals: aCollection.
	fail := self generateFailGoal.	"if newGoals succeeds we have to fail"

	self
		connect: (newGoals at: 2)
		to: fail
		class: RegexCuttedEpsilonTransition.
	remainingGoals isNil
		ifFalse:
			[self
				connect: notGoal
				to: (remainingGoals at: 1)
				class: RegexCuttedEpsilonTransition.
			end := remainingGoals at: 2]
		ifTrue:
			["we have to create a success-goal"

			end := self
				generateGoal:
					(SoulFactory current makeParser
						parse: 'if [ true ]'
						onError: [:a :b | self error: 'parse error in generated regexrule'])
						condition terms.
			self connect: notGoal to: end class: RegexCuttedEpsilonTransition].
	^Array with: notGoal with: end! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processOr: orTerm remainingConditions: aCollection
	
	| start end newGoals remainingGoals |
	newGoals := OrderedCollection new.
	orTerm termSequence terms
		do:
			[:termSequence | newGoals add: (self generateGoals: termSequence terms)].
	start := self generateGoal: OrderedCollection new.
	end := self generateGoal: OrderedCollection new.
	newGoals
		do:
			[:firstLast | 
			self
				connect: start
				to: (firstLast at: 1)
				class: RegexEpsilonTransition.
			self
				connect: (firstLast at: 2)
				to: end
				class: RegexEpsilonTransition].
	remainingGoals := self generateGoals: aCollection.
	remainingGoals isNil
		ifFalse:
			[self
				connect: end
				to: (remainingGoals at: 1)
				class: RegexTransition.
			end := remainingGoals at: 2].
	^Array with: start with: end! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processPlus: plusTerm remainingConditions: aCollection
	
	| firstLast first last remainingGoals |
	firstLast := self generateGoals: plusTerm termSequence terms.
	firstLast ifNil: [self error: 'plusTerm with empty arguments'].
	remainingGoals := self processManyOf: plusTerm remainingConditions:
			aCollection.
	first := firstLast at: 1.
	last := firstLast at: 2.
	remainingGoals isNil
		ifFalse:
			[self
				connect: (firstLast at: 2)
				to: (remainingGoals at: 1)
				class: RegexTransition.
			last := remainingGoals at: 2].
	^Array with: first with: last! !

!RegexTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processTimes: timesTerm remainingConditions: aCollection
	
	| times firstLast first last remainingGoals terms timesTerms |
	terms := timesTerm termSequence terms.
	terms size < 2
		ifTrue:
			[self error: 'times term requires at least 2 arguments, times and goals'].
	times := (terms at: 1) smalltalkValue.
	timesTerms := terms allButFirst: 1.
	firstLast := self generateGoals: timesTerms.
	first := firstLast at: 1.
	last := firstLast at: 2.
	2
		to: times
		do:
			[:i | 
			| generated |
			generated := self generateGoals: timesTerms.
			self
				connect: last
				to: (generated at: 1)
				class: RegexTransition.
			last := generated at: 2].
	remainingGoals := self generateGoals: aCollection.
	remainingGoals isNil
		ifFalse:
			[self
				connect: last
				to: (remainingGoals at: 1)
				class: RegexTransition.
			last := remainingGoals at: 2].
	^Array with: first with: last! !

!RegexTerm methodsFor: 'initialize-release' stamp: 'ReinoutStevens 1/12/2012 15:31'!
validateRegexTerms
	|size|
	size := self termSequence terms first termSequence terms size.
	((5 to: 5) includes: size)
		ifFalse:
			[self error:
					'goalTerm needs to have 5 arguments: transition, startworld, endworld, variables and a pathvariable'].! !

CompoundTerm subclass: #SpliceTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!SpliceTerm commentStamp: '<historical>' prior: 0!
SpliceCompound is used inside quotedcode splice a wrapped quotedcode inside a quotedcode

Subclasses must implement the following messages:
	converting
		smalltalkBlockString

!

!SpliceTerm methodsFor: 'quotedCode' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	"see comment in AbstractTerm"

	"When a compound term is used inside a quotedCode, this method gets invoked when the quotedcode is dowed and this means that we need to convert to our 	previous quotedcode representation. If we are not a compound that was unified with a quotedcode, then we simply write our textual representation"

	
	| string list index |
	string := WriteStream on: String new.
	list := functor soulDownIn: anEnv startAt: anIndex.
	(list isKindOf: ListTerm) ifFalse: [^self asString].
	index := 0.
	list
		listDo:
			[:listTerm | 
			string
				nextPutAll:
					(listTerm class = Variable
						ifTrue:
							[index := index + 1.
							(self termSequence terms at: index) asSourceStringIn: anEnv startAt:
									anIndex]
						ifFalse: [listTerm asSourceStringIn: anEnv startAt: anIndex])].
	index = self termSequence size
		ifFalse:
			[self error: 'Incorrect number of arguments to compoundterm in quotedcode'].
	^string contents! !

!SpliceTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockString
	"TODO"

	
	self subclassResponsibility! !

CompoundTerm subclass: #VarArgsCompoundTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!VarArgsCompoundTerm commentStamp: '<historical>' prior: 0!
This class represents a compoundterm that has a variable-length argument list. !

!VarArgsCompoundTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^-1! !

!VarArgsCompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	
	| newId |
	id = #asClause
		ifTrue: [newId := #asClause]
		ifFalse: [newId := #asTerm].
	functor prettyPrintAsFunctorOn: aStream scope: newId.
	aStream write: '@('.
	termsequence
		do: [:aterm | aterm prettyPrintOn: aStream scope: #asTerm]
		separatedBy: [aStream write: ','].
	aStream write: ')'! !

!VarArgsCompoundTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	functor printAsFunctorOn: aStream.
	aStream write: '@('.
	termsequence
		do: [:aterm | aterm printOn: aStream]
		separatedBy: [aStream write: ','].
	aStream write: ')'! !

!VarArgsCompoundTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
realmultiplicity
	
	^self termSequence size! !

!VarArgsCompoundTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: env startAt: anIndex
	"resolve as a normal clause: do lookup in repository and create a resolver  on it"

	
	| foundClauses |
	foundClauses := self definitionRepository
		varArgsAddressLookup:
			(self
				lexicalAddressInEnv: env
				startAt: anIndex
				ifAbsent:
					[^self
						clauseNotFoundIn: self definitionRepository
						inEnv: env
						startAt: anIndex]).
	^SoulFactory current makeResolver fromClause: self toClauses: foundClauses
		inEnv: env startAt: anIndex! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithVarArgsCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'AndyKellens 11/2/2011 09:53'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"unify a compound with a VarArgscompound => if the functors unify and our multiplicity is smaller or equal to the normal compound's multiplicity, then we unify the subterms and our last subterm unifies with a list of the remaining subterms of the normal compound"

	
	| otherTermSequence newListTerm |
	(self functor
		unifyWith: aCompound functor
		inEnv: anEnv
		myIndex: myIndex
		hisIndex: hisIndex
		inSource: inSource) & (aCompound multiplicity >= self realmultiplicity)
		ifTrue:
			[otherTermSequence := aCompound termSequence.
			self realmultiplicity = 1
				ifTrue:
					[newListTerm := SoulFactory current makeList
						termSequence: aCompound termSequence.
					(newListTerm
						unifyWith: (termsequence terms at: 1)
						inEnv: anEnv
						myIndex: hisIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)) ifFalse: [^false]]
				ifFalse:
					[1
						to: self realmultiplicity - 1
						do:
							[:index | 
							((otherTermSequence terms at: index)
								unifyWith: (termsequence terms at: index)
								inEnv: anEnv
								myIndex: hisIndex
								hisIndex: myIndex
								inSource: (inSource xor: true)) ifFalse: [^false]].
					(SoulFactory current makeList
						termSequence:
							(TermSequence
								terms:
									(aCompound termSequence terms
										copyFrom: self realmultiplicity
										to: aCompound multiplicity)))
						unifyWith: (termsequence terms at: self realmultiplicity)
						inEnv: anEnv
						myIndex: hisIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)].
			^true]
		ifFalse: [^false]! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'AndyKellens 11/2/2011 09:53'!
unifyWithListTerm: aListTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| otherTermSequence newListTerm collection |
	collection := aListTerm collectIn: anEnv startAt: hisIndex.
	(self functor
		unifyWith: aListTerm functor
		inEnv: anEnv
		myIndex: myIndex
		hisIndex: hisIndex
		inSource: inSource) & (collection size >= self realmultiplicity)
		ifTrue:
			[otherTermSequence := TermSequence
				terms: (collection collect: [:t | t soulUp]).
			self realmultiplicity = 1
				ifTrue:
					[newListTerm := SoulFactory current makeList termSequence: otherTermSequence.
					newListTerm
						unifyWith: (termsequence terms at: 1)
						inEnv: anEnv
						myIndex: hisIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]
				ifFalse:
					[1
						to: self realmultiplicity - 1
						do:
							[:index | 
							((otherTermSequence terms at: index)
								unifyWith: (termsequence terms at: index)
								inEnv: anEnv
								myIndex: hisIndex
								hisIndex: myIndex
								inSource: (inSource xor: true)) ifFalse: [^false]].
					(SoulFactory current makeList
						termSequence:
							(TermSequence
								terms:
									(collection
										copyFrom: self realmultiplicity
										to: collection size)))
						unifyWith: (termsequence terms at: self realmultiplicity)
						inEnv: anEnv
						myIndex: hisIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)].
			^true]
		ifFalse: [^false]! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithVarArgsCompoundTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject
		unifyWithVarArgsCompoundTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^super unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex
		hisIndex: hisIndex inSource: inSource! !

!VarArgsCompoundTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithVarArgsCompoundTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

ResolvableTerm subclass: #Cut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!Cut commentStamp: '<historical>' prior: 0!
Represents the Prolog cut.

!

!Cut methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"See comment in AbstractTerm"

	
	^aVisitor cutVisit: self! !

!Cut methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isCut
	
	^true! !

!Cut methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: scope in: env startAt: idx
	"See comment in AbstractTerm"

	
	aStream write: '!!'! !

!Cut methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"See comment in AbstractTerm"

	
	aStream write: self class name , ' new'! !

!Cut methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"See comment in AbstractTerm"

	
	aStream write: '!!'! !

!Cut methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: anEnv startAt: anIndex
	"Resolving a cut means cutting of the callstack to avoid backtracking and making an empty resolver to allow further evaluation"

	
	anEnv callStack cut.
	^NativeResolver
		onBlock:
			[:foo | 
			ResolutionResult
				condition:
					(SoulFactory current makeTermSequence
						terms: OrderedCollection new
						containsCut: false)
				rollbackInfo: anEnv rollbackInfo
				conditionsEnvIndex: anEnv currentVariableIndex
				cutState: false]
		withArguments:
			((OrderedCollection new)
				add: #foo;
				yourself)! !

!Cut methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self! !

!Cut methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"double dispatch the unification of the term"

	
	^aTerm
		unifyWithCut: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: (inSource xor: true)! !

!Cut methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCut: aCut inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^true! !

!Cut methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithCut: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!ResolvableTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
clauseNotFoundIn: repository inEnv: env startAt: anIndex
	
	(self
		dynamicPredicate: (self predicateInEnv: env startAt: anIndex)
		inEnv: env
		startAt: anIndex)
		ifTrue:
			[^SoulFactory current makeNativeResolver
				onBlock: [:res | res]
				withArguments: #(nil)]
		ifFalse:
			[self
				error:
					'Cannot resolve ' , self asString , ' in repository ' , repository name]! !

!ResolvableTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
dynamicPredicate: aPredicate inEnv: anEnv startAt: anIndex
	
	| evaluator result |
	(self definitionRepository addressForPredicate: 'dynamic')
		ifNil: [^false]
		ifNotNil:
			[evaluator := SoulFactory current evaluatorForLogicQuery
				eval: 'if dynamic(#' , aPredicate , ')'
				in: self definitionRepository.
			result := evaluator nextResult.
			^result success]! !

!ResolvableTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
lexicalAddress
	
	^lexicalAddress! !

!ResolvableTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
lexicalAddress: anObject
	
	lexicalAddress := anObject.
	lexicalAddress = 0 ifTrue: [self halt]! !

!ResolvableTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
lexicalAddressInEnv: anEnv startAt: anIndex ifAbsent: aBlock
	
	^self
		lexicalAddressInEnv: anEnv
		startAt: anIndex
		repository: self definitionRepository
		ifAbsent: aBlock! !

!ResolvableTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
lexicalAddressInEnv: anEnv startAt: anIndex repository: aRep ifAbsent: aBlock
	"Lookup the address of the predicate of this clause in the database"

	
	| addr index |
	addr := self lexicalAddress.
	^addr
		ifNil:
			[index := aRep
				addressForPredicate: (self predicateInEnv: anEnv startAt: anIndex).
			index
				ifNil: aBlock
				ifNotNil: [index]]
		ifNotNil: [addr]! !

!ResolvableTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: env startAt: anIndex
	"resolve as a normal term: do lookup in repository and create a resolver  on it"

	
	| foundClauses |
	foundClauses := self definitionRepository
		normalAddressLookup:
			(self
				lexicalAddressInEnv: env
				startAt: anIndex
				ifAbsent:
					[^self
						clauseNotFoundIn: self definitionRepository
						inEnv: env
						startAt: anIndex])
		multiplicity: self multiplicity.
	^SoulFactory current makeResolver fromClause: self toClauses: foundClauses
		inEnv: env startAt: anIndex! !

ResolvableTerm subclass: #ResultMessageTerm
	instanceVariableNames: 'messageTerm resultTerm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!ResultMessageTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
message: message result: result
	
	^self new message: message result: result! !

!ResultMessageTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: visitor
	
	^visitor resultMessageTermVisit: self! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
arguments
	
	^messageTerm termSequence terms allButFirst: 1! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
functor
	
	^messageTerm functor! !

!ResultMessageTerm methodsFor: 'misc' stamp: 'sp 24/10/11 15:26'!
lexicalAddressInEnvNoEquals: anEnv startAt: idx ifAbsent: absentBlock
	
	^messageTerm lexicalAddressInEnv: anEnv startAt: idx ifAbsent: absentBlock! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
message
	
	^messageTerm! !

!ResultMessageTerm methodsFor: 'initialization' stamp: 'sp 24/10/11 15:26'!
message: message result: result
	
	messageTerm := message.
	resultTerm := result! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^messageTerm multiplicity + 1! !

!ResultMessageTerm methodsFor: 'misc' stamp: 'sp 24/10/11 15:26'!
predicate
	
	^(messageTerm predicateInEnv: nil startAt: nil) , '='! !

!ResultMessageTerm methodsFor: 'misc' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: idx
	
	^(messageTerm predicateInEnv: anEnv startAt: idx) , '='! !

!ResultMessageTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:02'!
prettyPrintOn: aStream scope: scope
	
	messageTerm prettyPrintOn: aStream scope: scope.
	aStream withAttribute:(TextColor green) do:[
	aStream nextPutAll: ' = '].
	resultTerm prettyPrintOn: aStream scope: scope.
! !

!ResultMessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	messageTerm functor printAsPrologFunctorOn: aStream scope: #asTerm in: env
		startAt: idx.
	aStream write: '('.
	messageTerm termSequence printAsPrologOn: aStream scope: #asTerm in: env
		startAt: idx.
	aStream write: ')'! !

!ResultMessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	
	aStream nextPutAll: self class name , ' message: ('.
	messageTerm printForCompileOn: aStream.
	aStream nextPutAll: ') result: ('.
	resultTerm printForCompileOn: aStream.
	aStream nextPutAll: ')'! !

!ResultMessageTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	messageTerm printOn: aStream.
	aStream nextPutAll: ' = '.
	resultTerm printOn: aStream! !

!ResultMessageTerm methodsFor: 'As yet unclassified' stamp: 'sp 24/10/11 15:26'!
receiver
	
	^messageTerm termSequence terms first! !

!ResultMessageTerm methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	"see comment in AbstractTerm"

	
	self definitionRepository: aRepository.
	messageTerm recursiveDefinitionRepository: aRepository.
	resultTerm recursiveDefinitionRepository: aRepository! !

!ResultMessageTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: env startAt: anIndex
	"resolve as a normal term: do lookup in repository and create a resolver  on it"

	
	| foundClauses multiplicity address |
	multiplicity := self multiplicity.
	address := self
		lexicalAddressInEnv: env
		startAt: anIndex
		ifAbsent:
			[multiplicity := multiplicity - 1.
			self
				lexicalAddressInEnvNoEquals: env
				startAt: anIndex
				ifAbsent:
					[^self
						clauseNotFoundIn: self definitionRepository
						inEnv: env
						startAt: anIndex]].
	foundClauses := self definitionRepository normalAddressLookup: address
		multiplicity: multiplicity.
	^SoulFactory current makeResolver fromClause: self toClauses: foundClauses
		inEnv: env startAt: anIndex! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
result
	
	^resultTerm! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
selector
	
	^messageTerm functor name! !

!ResultMessageTerm methodsFor: 'accessing' stamp: 'AndyKellens 11/2/2011 09:53'!
termSequence
	
	^SoulFactory current makeTermSequence
		terms:
			((messageTerm termSequence terms copy)
				add: resultTerm;
				yourself)! !

!ResultMessageTerm methodsFor: 'unification' stamp: 'AndyKellens 10/28/2011 13:33'!
unifyWith: aTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"double dispatch the unification of the term"
	
	^aTerm unifyWithResultMessageTerm: self inEnv: anEnv myIndex: targetIndex
		hisIndex: sourceIndex inSource: inSource! !

!ResultMessageTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^(self result
		unifyWith: true soulUp
		inEnv: anEnv
		myIndex: myIndex
		hisIndex: hisIndex
		inSource: inSource)
		and:
			[self message unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex:
					myIndex hisIndex: hisIndex inSource: inSource]! !

!ResultMessageTerm methodsFor: 'unification' stamp: 'AndyKellens 10/28/2011 13:33'!
unifyWithResultMessageTerm: aRM inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	^(messageTerm
		unifyWith: aRM message
		inEnv: anEnv
		myIndex: sourceIndex
		hisIndex: targetIndex
		inSource: inSource)
		and:
			[resultTerm
				unifyWith: aRM result
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource]! !

!ResultMessageTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithResultMessageTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

ResolvableTerm subclass: #SymbiosisTerm
	instanceVariableNames: 'source block variables content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!SymbiosisTerm commentStamp: '<historical>' prior: 0!
This abstract class is the superclass of all logic terms that deal with symbiosis with Smalltalk. It groups their common behaviour

Subclasses must implement the following messages:
	private
		constructBlock
		parse:
		parseVariables

Instance Variables:
	block	<BlockClosure | MessageChannel>	description of block
	content	<RBProgramNode>	description of content
	source	<AbstractItem | ByteFieldDescriptor | CCompoundType | (Collection of: Character) | MC_FileBTree | PostgreSQLQueryResultList | RBProgramNode | SmaCCRHS | SOULResults | Stream | TermSequence>	description of source
	variables	<(OrderedCollection of: Variable)>	description of variables

!

SymbiosisTerm subclass: #QuotedCodeTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!QuotedCodeTerm commentStamp: '<historical>' prior: 0!
This class represents the quotedcode in a logic program!

!QuotedCodeTerm class methodsFor: 'signal constants' stamp: 'sp 24/10/11 15:26'!
errorSignal
	
	^SoulParseError! !

!QuotedCodeTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor quotedCodeTermVisit: self! !

!QuotedCodeTerm methodsFor: 'quotedCode' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	
	^self evaluateIn: anEnv startAt: anIndex! !

!QuotedCodeTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
compactedContent
	
	| newContent previous string |
	newContent := OrderedCollection new.
	previous := WriteStream on: String new.
	content
		do:
			[:el | 
			string := el smalltalkBlockString.
			string size = 3
				ifTrue: [previous nextPut: (string at: 2)]
				ifFalse:
					[newContent add: (QuotedCodeConstant name: previous contents).
					previous := WriteStream on: String new.
					newContent add: el]].
	previous contents size > 0
		ifTrue: [newContent add: (QuotedCodeConstant name: previous contents)].
	^newContent! !

!QuotedCodeTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
constructBlock
	"This method constructs the SmalltalkBlock to be executed if the  
	quotedcode needs to be downed to a string"

	
	| sourceCode blockNode temporaries statements compactedContent |
	compactedContent := self compactedContent.
	sourceCode := WriteStream on: String new.
	sourceCode nextPutAll: '[:env :envIndex '.
	variables size > 2
		ifTrue:
			[(3 to: variables size)
				do:
					[:index | sourceCode nextPutAll: ':arg' , (variables at: index) name , ' ']].
	sourceCode nextPutAll: ' | '.
	compactedContent isEmpty
		ifTrue: [sourceCode nextPutAll: '''''']
		ifFalse:
			[compactedContent
				do: [:el | sourceCode nextPutAll: el smalltalkBlockString]
				separatedBy: [sourceCode nextPutAll: ' , ']].
	sourceCode nextPutAll: ']'.
	variables size > 2
		ifTrue:
			[blockNode := RBParser parseExpression: sourceCode contents.
			statements := blockNode body statements asOrderedCollection.
			temporaries := blockNode body temporaries asOrderedCollection.
			(3 to: variables size)
				do:
					[:index | 
					temporaries
						add: (RBVariableNode named: 'soulvar' , (variables at: index) name)].
			(3 to: variables size)
				do:
					[:index | 
					statements
						addFirst:
							(RBParser
								parseExpression:
									'soulvar' , (variables at: index) name , ' := ' , 'arg'
										, (variables at: index) name ,
											' asSourceStringIn: env startAt: envIndex.')].
			blockNode body statements: statements.
			blockNode body temporaries: temporaries.
			block := self compilerForSmalltalkTermBlocks
				evaluate: blockNode formattedCode]
		ifFalse:
			[block := self compilerForSmalltalkTermBlocks evaluate: sourceCode contents]! !

!QuotedCodeTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
parse
	
	content := source isEmpty
		ifTrue: [#()]
		ifFalse:
			[SoulFactory current makeQuotedCodeParser
				parse: source
				onError:
					[:text :pos | self error: 'Parser error: ' , text , 'at: ' , pos asString]]! !

!QuotedCodeTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
parseVariables
	
	| dict visitor |
	variables := OrderedCollection new.
	dict := Dictionary new.
	visitor := SoulFactory current makeNamedVariableVisitor
		doBlock:
			[:var | 
			dict
				at: var name
				ifAbsentPut:
					[variables addLast: var.
					var]].
	content do: [:each | each accept: visitor].
	variables addFirst: nil.
	variables addFirst: nil.
	variables := variables asArray! !

!QuotedCodeTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:03'!
prettyPrintOn: aStream scope: id

	aStream withAttributes:(Array with:(TextColor color:(Color r:0 g:0 b:256)) with:(TextEmphasis bold)) do:[aStream write: '{'.].
	super prettyPrintOn: aStream scope: id.
	aStream withAttributes:(Array with:(TextColor color:(Color r:0 g:0 b:256)) with:(TextEmphasis bold)) do:[aStream write: '}'.].! !

!QuotedCodeTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' source: '.
	source printOn: aStream! !

!QuotedCodeTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: '{'.
	aStream write: source.
	aStream write: '}'! !

!QuotedCodeTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: anEnv startAt: anIndex
	
	| term visitor |
	term := (SoulFactory current makeParser
		parse: source
		onError: [self error: 'cannot parse contents as a soul term']) clauses
		first conclusion.
	visitor := FixVisitor new addVariables: self variables.
	term accept: visitor.
	term definitionRepository: self definitionRepository.
	^term resolveIn: anEnv startAt: anIndex! !

!QuotedCodeTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"DownedQuotedCode represents the QuotedCode construct on the  
	smalltalk-level. We also down our contents to allow an up later on"

	
	| string newcontent newvariables |
	string := self asSourceStringIn: anEnv startAt: anIndex.
	newvariables := variables
		collect:
			[:el | 
			el
				ifNil: [nil]
				ifNotNil: [(el soulDownIn: anEnv startAt: anIndex) soulUp]].
	newcontent := content
		collect: [:el | (el soulDownIn: anEnv startAt: anIndex) soulUp].
	^DownedQuotedCode fromString: string variables: newvariables content:
			newcontent block: block! !

!QuotedCodeTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	| string newvariables newcontent |
	string := self asSourceStringIn: anEnv startAt: anIndex.
	newvariables := variables
		collect:
			[:el | 
			el
				ifNil: [nil]
				ifNotNil: [el transitiveLookupIn: anEnv startAt: anIndex]].
	newcontent := content
		collect: [:el | el transitiveLookupIn: anEnv startAt: anIndex].
	^QuotedCodeTerm source: string variables: newvariables content: newcontent
		block: block! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithQuotedCodeTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'AndyKellens 11/2/2011 09:53'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"a quotedCode unifies with a compound in the following way:
	{a b c d} <-> <a,b,c,d>
	{a ?b c ?d} <-> <a,?b,c,?d>(?b,?d)"

	
	| vars otherTerms |
	vars := self variables.
	vars size == aCompound multiplicity
		ifTrue:
			[otherTerms := aCompound termSequence terms.
			vars
				with: otherTerms
				do:
					[:myTerm :otherTerm | 
					(otherTerm
						unifyWith: myTerm
						inEnv: anEnv
						myIndex: hisIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)) ifFalse: [^false]].
			^aCompound functor
				unifyWith:
					(SoulFactory current makeList
						termSequence:
							(TermSequence new terms: (content collect: [:each | each quotedCodeToList])))
				inEnv: anEnv
				myIndex: hisIndex
				hisIndex: myIndex
				inSource: (inSource xor: true)]
		ifFalse: [^false]! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"TODO: how to unify quotedCode? Currently we, compare strings. An  
	alternative is in the comment"

	"a quotedCode unifies with another quotedCode if all subterms unify  
	| index succes |  
	succes _ true.  
	index _ 1.  
	self terms size == aQCTerm terms size  
	ifFalse: [^ false].  
	aQCTerm terms  
	do: [:otherTerm |  
	succes  
	ifTrue: [succes _ otherTerm  
	unifyWith: (content at: index)  
	inEnv: anEnv].  
	index _ index + 1].  
	^ succes"

	
	^(self asSourceStringIn: anEnv startAt: myIndex)
		= (aQCTerm asSourceStringIn: anEnv startAt: hisIndex)! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: aSTTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^(aSTTerm evaluateIn: anEnv startAt: targetIndex) soulUp
		unifyWithQuotedCodeTerm: self
		inEnv: anEnv
		myIndex: targetIndex
		hisIndex: sourceIndex
		inSource: inSource not! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^anObject unifyWithQuotedCodeTerm: self inEnv: anEnv myIndex: targetIndex
		hisIndex: sourceIndex inSource: inSource! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	| value |
	value := anObject smalltalkValue.
	value ifNil: [^false].
	^(self asSourceStringIn: anEnv startAt: sourceIndex) match: value! !

!QuotedCodeTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"a quotedCode unifies with a variable by binding to this variable or 
	unifying with it's binding"

	
	| val |
	val := anEnv lookup: aVariable startAt: hisIndex.
	val isNil
		ifTrue:
			[anEnv bind: aVariable to: self startAt: hisIndex valueStartAt: myIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithQuotedCodeTerm: self
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: myIndex
				inSource: (inSource xor: true)]! !

QuotedCodeTerm subclass: #TemplateQueryTerm
	instanceVariableNames: 'type parsetrees queries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!TemplateQueryTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
source: aString type: aType
	
	^(self new)
		type: aType;
		source: aString! !

!TemplateQueryTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
source: string variables: newvariables content: newcontent block: block parsetrees: newparsetree queries: queries type: type
	
	^self new source: string variables: newvariables content: newcontent
		block: block parsetrees: newparsetree queries: queries type: type! !

!TemplateQueryTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor templateQueryTermVisit: self! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
asParseTrees
	"This method is only called after the repository of clauses has been loaded. It parses the contents and builds a set of logic parsetrees of the source, using the DCG grammar rules in the repository."

	
	| results trees |
	results := (SoulFactory current evaluatorForTemplateParsing
		eval:
			'if parse(' , type functor asString , ',[''' , source asString ,
					'''],?result)'
		in: SoulFactory parsingRepository) allResults.
	results size = 0 ifTrue: [self error: 'Parse error in quotedcode'].
	trees := OrderedCollection new.
	results
		do: [:value | trees add: (value accept: CopyingVisitor new)]
		forVariableNames: (OrderedCollection with: 'result').
	^trees! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
asQueries
	
	| results thesequeries parsetreerepresentation queryclauses conversionQueryString associations |
	thesequeries := OrderedCollection new.
	self parsetrees
		do:
			[:parsetree | 
			parsetreerepresentation := (parsetree accept: VariableWrapperVisitor new)
				asString.
			associations := self associationsForConversionQueryString.
			conversionQueryString := self conversionQueryStringForParseTree:
					parsetreerepresentation.
			results := (SoulFactory current evaluatorForTemplateCompilation
				eval: conversionQueryString
				in: SoulFactory compilingRepository
				withAssociations: associations) allResults.
			results size = 0 ifTrue: [self error: 'conversion error in quotedcode'].
			results
				do:
					[:query :annotation | 
					| evaluator queryCopy annotationCopy |
					queryclauses := OrderedCollection new.
					query listDo: [:aTerm | queryclauses add: aTerm].	"not so nice solution: in order to overcome the shared compound term instances problem in these queries, we create an entirely new query through parsing"

					evaluator := SoulFactory current evaluatorForTemplateCompilation
						eval: (Query condition: (TermSequence terms: queryclauses)) asString
						in: SoulFactory compilingRepository.
					queryCopy := evaluator query condition
						accept: VariableUnwrapperVisitor new.
					annotationCopy := annotation accept: CopyingVisitor new.
					thesequeries add: (Array with: queryCopy with: annotationCopy)]
				forVariableNames: (OrderedCollection with: 'query' with: 'annotation')].
	^thesequeries! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
associationsForConversionQueryString
	
	| arguments associations |
	arguments := self type termSequence terms.
	self hasInterpretationArgument
		ifFalse:
			[arguments size = 1
				ifTrue: [associations := Array with: #resultTerm -> arguments first]
				ifFalse: [self error: 'incorrect amount of arguments']]
		ifTrue:
			[arguments size = 2
				ifTrue:
					[associations := Array
						with: #resultTerm -> arguments first
						with: #interpretationTerm -> (arguments at: 2)]
				ifFalse: [self error: 'incorrect amount of arguments']].
	^associations! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
conversionQueryStringForParseTree: parsetreerepresentation
	"Er wordt verwacht dat het resultaat van de template-resolutie ?result unificeert met ?resultTerm. Deze resolutie zal resultaten uit de Eclipse workspace ophalen, tenzij ?result al gebonden was aan ?resultTerm voor de resolutie begon. Om performantieoverwegingen wordt de conditie ?result equals: ?resultTerm daarom vooraan de gegenereerde query geplakt. Op methods en statements  zijn namelijk geen unificaties gedefinieerd die ervoor zouden kunnen zorgen dat er andere bindingen in de Eclipse workspace gevonden kunnen worden voor ?result die unificeren met ?resultTerm, maar er niet gelijk aan zijn. Op Eclipse expressions is dit wel gebeurd.

Concreet is het performantieverschil merkbaar bij de volgende query:
?s isStatement,
jtStatement(?s){return ?e;}
"

	
	| conversionQueryString functorName toBeLookedUpEvenIfGround |
	functorName := self type functor name.
	toBeLookedUpEvenIfGround := #(#jtExpression #jtClassDeclaration #jtInterfaceDeclaration #jtMethodDeclaration).
	conversionQueryString := 'if ' , parsetreerepresentation asString ,
			' templateUnderInterpretation: ?interpretationFooBar123 compilesTo: ?tempquery forResult: ?resultFooBar123 : ?annotation '.
	conversionQueryString := conversionQueryString
		,
			((toBeLookedUpEvenIfGround includes: functorName)
				ifTrue:
					[', append('
						,
							(self hasInterpretationArgument
								ifTrue:
									['<?interpretationFooBar123 equals: ?interpretationTerm | ?tempquery>']
								ifFalse: ['?tempquery']) ,
							', <?resultFooBar123 equals: ?resultTerm>, ?query)']
				ifFalse:
					[', ?query equals: <'
						,
							(self hasInterpretationArgument
								ifTrue: ['?interpretationFooBar123 equals: ?interpretationTerm,']
								ifFalse: ['']) ,
							' ?resultFooBar123 equals: ?resultTerm | ?tempquery>']).
	^conversionQueryString! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
hasInterpretationArgument
	
	^self type termSequence terms size = 2! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
parse
	"Delay parsing, if necessary, to after the loading of the entire repository"

	"Soul.Factory doAfterRepositoryLoad: [self setParseTree]."

	
	parsetrees := self asParseTrees.
	queries := self asQueries.
	^super parse! !

!TemplateQueryTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
parsetrees
	
	^parsetrees! !

!TemplateQueryTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	
	type prettyPrintOn: aStream scope: id.
	super prettyPrintOn: aStream scope: id! !

!TemplateQueryTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' source: '.
	source printOn: aStream.
	aStream write: ' type: '.
	type printOn: aStream! !

!TemplateQueryTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	type printOn: aStream.
	super printOn: aStream! !

!TemplateQueryTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
queries
	
	^queries! !

!TemplateQueryTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	
	self definitionRepository: aRepository.
	self queries
		do:
			[:aQuery | aQuery do: [:i | i recursiveDefinitionRepository: aRepository]]! !

!TemplateQueryTerm methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: anEnv startAt: anIndex
	
	| resolveBlock |
	resolveBlock := [:aQuery | 
	ResolutionResult
		condition: (aQuery at: 1)
		rollbackInfo: anEnv rollbackInfo
		conditionsEnvIndex: anIndex
		cutState: false].
	^NativeResolver
		onBlock: resolveBlock
		withArguments: self queries! !

!TemplateQueryTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"DownedQuotedCode represents the QuotedCode construct on the  
	smalltalk-level. We also down our contents to allow an up later on"

	
	| string newcontent newvariables newparsetrees newqueries |
	string := self asSourceStringIn: anEnv startAt: anIndex.
	newvariables := variables
		collect:
			[:el | 
			el
				ifNil: [nil]
				ifNotNil: [el soulDownIn: anEnv startAt: anIndex]].
	newcontent := content
		collect: [:el | (el soulDownIn: anEnv startAt: anIndex) soulUp].
	newparsetrees := parsetrees
		collect: [:tree | (tree soulDownIn: anEnv startAt: anIndex) soulUp].
	newqueries := queries
		collect:
			[:tree | 
			Array
				with: ((tree at: 1) transitiveLookupIn: anEnv startAt: anIndex) soulUp
				with: (tree at: 2)].
	^DownedTemplateQuery fromString: string variables: newvariables content:
			newcontent block: block parsetrees: newparsetrees queries: newqueries
		type: type! !

!TemplateQueryTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
source: aString variables: aVarCollection content: aContent block: aBlock parsetrees: aParsetree queries: aQueries type: aType
	
	self source: aString variables: aVarCollection content: aContent block:
			aBlock.
	parsetrees := aParsetree.
	queries := aQueries.
	type := aType! !

!TemplateQueryTerm methodsFor: 'reification' stamp: 'AndyKellens 11/2/2011 09:53'!
transitiveLookupIn: anEnv startAt: anIndex
	
	| string newvariables newcontent newparsetrees newqueries |
	string := self asSourceStringIn: anEnv startAt: anIndex.
	newvariables := variables
		collect:
			[:el | 
			el
				ifNil: [nil]
				ifNotNil: [el transitiveLookupIn: anEnv startAt: anIndex]].
	newcontent := content
		collect: [:el | el transitiveLookupIn: anEnv startAt: anIndex].
	newparsetrees := parsetrees
		collect:
			[:tree | (tree transitiveLookupIn: anEnv startAt: anIndex) soulUp].
	newqueries := queries
		collect:
			[:tree | 
			Array
				with: ((tree at: 1) transitiveLookupIn: anEnv startAt: anIndex) soulUp
				with: (tree at: 2)].
	^SoulFactory current makeTemplateQuery source: string variables: newvariables
		content: newcontent block: block parsetrees: newparsetrees queries:
			newqueries type: type! !

!TemplateQueryTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
type
	"The type of the quotedcode"

	
	^type! !

!TemplateQueryTerm methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
type: aType
	"initalize the type of the quotedcode"

	
	type := aType! !

SymbiosisTerm subclass: #SmalltalkTerm
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!SmalltalkTerm commentStamp: '<historical>' prior: 0!
This class represents a smalltalkterm in a logic program!

!SmalltalkTerm methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor smalltalkTermVisit: self! !

!SmalltalkTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
asDownedSourceStringIn: anEnv startAt: anIndex
	
	| variableValues downedSource |
	variableValues := self variables
		collect: [:aVariable | aVariable soulDownIn: anEnv startAt: anIndex].
	downedSource := source copy.
	self variables
		doWithIndex:
			[:aVariable :index | 
			downedSource := downedSource
				copyReplaceAll: '?' , aVariable name
				with: (variableValues at: index) displayString].
	^downedSource! !

!SmalltalkTerm methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asSourceString
	
	self shouldNotImplement! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
constructBlock
	"This method constructs the SmalltalkBlock to be executed if the  
	SmalltalkTerm needs to execute"

	
	| sourceCode blockNode temporaries statements |
	sourceCode := WriteStream on: String new.
	sourceCode nextPutAll: '[:env :envIndex '.
	variables size > 2
		ifTrue:
			[(3 to: variables size)
				do:
					[:index | sourceCode nextPutAll: ':arg' , (variables at: index) name , ' ']].
	sourceCode nextPutAll: ' | '.
	sourceCode nextPutAll: content formattedCode.
	sourceCode nextPutAll: ']'.
	variables size > 2
		ifTrue:
			[blockNode := RBParser parseExpression: sourceCode contents.
			statements := blockNode body statements asOrderedCollection.
			temporaries := blockNode body temporaries asOrderedCollection.
			(3 to: variables size)
				do:
					[:index | 
					temporaries
						add: (RBVariableNode named: 'soulvar' , (variables at: index) name)].
			(3 to: variables size)
				do:
					[:index | 
					statements
						addFirst:
							(RBParser
								parseExpression:
									'soulvar' , (variables at: index) name , ' := '
										, (variables at: index) smalltalkBlockDowningString , '.')].
			blockNode body statements: statements.
			blockNode body temporaries: temporaries.
			block := self compilerForSmalltalkTermBlocks
				evaluate: blockNode formattedCode]
		ifFalse:
			[block := self compilerForSmalltalkTermBlocks evaluate: sourceCode contents]! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
isBlock
	
	^false! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
isLiteral
	
	^content statements size = 1
		and: [content statements first isKindOf: RBLiteralNode]! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
isMethod
	
	^false! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
literalValue
	
	^content statements first value! !

!SmalltalkTerm methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
parse
	
	content := SoulFactory current makeSmalltalkTermParser
		parseExpression: source
		onError:
			[:text :pos | self error: 'Parser error: ' , text , 'at: ' , pos asString].
	content parent: self! !

!SmalltalkTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
parseVariables
	
	| visitor |
	visitor := RBSoulVariableExtractorVisitor new.
	content acceptVisitor: visitor.
	variables := visitor variables.
	variables addFirst: nil.
	variables addFirst: nil.
	variables := variables asArray! !

!SmalltalkTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:03'!
prettyPrintOn: aStream scope: id
	
	aStream
		withAttributes:
			(Array
				with:(TextColor color:(Color r:0 g:0 b:256))
				with: TextEmphasis bold) do:[
	aStream write: '['.].
	super prettyPrintOn: aStream scope: id.
		aStream
		withAttributes:
			(Array
				with:(TextColor color:(Color r:0 g:0 b:256))
				with: TextEmphasis bold) do:[
	aStream write: ']'.].
	! !

!SmalltalkTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	self isLiteral
		ifTrue:
			[self literalValue printAsPrologOn: aStream scope: id in: env startAt: idx]
		ifFalse:
			["Rather than spitting out an error, this code will simply output the source of the Smalltalk term as a string in a compound term for later manual translation."

			aStream write: 'smalltalkterm /*' , source , '*/']! !

!SmalltalkTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' source: '.
	source printOn: aStream! !

!SmalltalkTerm methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: '['.
	aStream write: source.
	aStream write: ']'! !

!SmalltalkTerm methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: anEnv startAt: anIndex
	
	| resolveBlock resolveBlockArgs result |
	resolveBlockArgs := OrderedCollection new.
	resolveBlockArgs add: variables.
	resolveBlock := [:arg | 
	arg at: 1 put: anEnv.
	arg at: 2 put: anIndex.
	result := [block valueWithArguments: arg]
		on: Exception
		do:
			[:ex | 
			(((ex isKindOf: Notification) or: [ex isKindOf: SoulRuntimeException]) or:[ex isKindOf: Halt])
				ifTrue: [ex pass].
			self error:
					'Smalltalk clauses should always evaluate to true or false!! (throws an error)'].
	arg at: 1 put: nil.
	arg at: 1 put: nil.
	((result isKindOf: Boolean) or: [result isKindOf: Maybe])
		ifFalse:
			[self error: 'Smalltalk clauses should always evaluate to true or false!!'].
	result
		ifTrue:
			[ResolutionResult
				condition: SoulFactory current makeTermSequence new
				rollbackInfo: anEnv rollbackInfo
				conditionsEnvIndex: nil
				cutState: false]].
	^NativeResolver onBlock: resolveBlock withArguments: resolveBlockArgs! !

!SmalltalkTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"the downed version of a SmallTalkTerm is the result of it's evaluation"

	
	^self evaluateIn: anEnv startAt: anIndex! !

!SmalltalkTerm methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'AndyKellens 7/30/2012 16:50'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	^aTerm
		unifyWithSmalltalkTerm: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: compound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object if my result of evaluation unifies with this  object"

	
	| object |
	object := (self evaluateIn: anEnv startAt: myIndex) soulUp.
	object == self
		ifTrue: [^false]
		ifFalse:
			[^object unifyWithCompoundTerm: compound inEnv: anEnv myIndex: myIndex
				hisIndex: hisIndex inSource: inSource]! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithDelayedVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a delayed variable by binding myself to it or unifying  
	with it's value"

	
	| val |
	val := anEnv lookup: aVariable startAt: hisIndex.
	val isNil
		ifTrue:
			[anEnv bind: aVariable to: self startAt: hisIndex valueStartAt: myIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithSmalltalkTerm: self
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: myIndex
				inSource: (inSource xor: true)]! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^(self evaluateIn: anEnv startAt: myIndex) soulUp
		unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex:
			hisIndex inSource: inSource! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: listTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object if my result of evaluation unifies with this  
	object"

	
	^(self evaluateIn: anEnv startAt: myIndex) soulUp unifyWithListTerm:
			listTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource:
			inSource! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^(self evaluateIn: anEnv startAt: sourceIndex) soulUp
		unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: sourceIndex
		hisIndex: targetIndex inSource: inSource! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with another SmallTalkTerm if my result and his result unify."

	
	^(self evaluateIn: anEnv startAt: myIndex) soulUp unifyWithSmalltalkTerm:
			anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource:
			inSource! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object if my result of evaluation unifies with this  
	object"

	
	^(self evaluateIn: anEnv startAt: myIndex) soulUp unifyWithUppedObject:
			anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource:
			inSource! !

!SmalltalkTerm methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a variable by binding my value to it or unifying with it's 
	value"

	
	^self unifyWith: aVariable inEnv: anEnv myIndex: myIndex hisIndex:
			hisIndex inSource: inSource! !

!SymbiosisTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
source: aString
	
	^self new source: aString! !

!SymbiosisTerm class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
source: aString variables: aCollection content: aContent block: aBlock
	
	^self new source: aString variables: aCollection content: aContent block:
			aBlock! !

!SymbiosisTerm methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConditionFor: aCounter
	
	| tokens1var tokens2var equals |
	tokens1var := SoulFactory current makeVariable
		name: 'tokens' , aCounter asString.
	tokens2var := SoulFactory current makeVariable
		name: 'tokens' , (aCounter + 1) asString.
	equals := SoulFactory current makeCompound
		functor: (Constant name: #equals)
		termSequence:
			(TermSequence terms: (OrderedCollection with: tokens1var with: tokens2var)).
	^SoulFactory current makeCompound
		functor: (Constant name: #and)
		termSequence:
			(TermSequence terms: (OrderedCollection with: self with: equals))! !

!SymbiosisTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
block
	"This is the block to be evaluated when trying to resolve a symbiosisterm"

	
	^block! !

!SymbiosisTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
compilerForSmalltalkTermBlocks
	"TODO: Find a cleaner way to do make this Squeak compatible. Pass by the MLI which already has VW and Squeak-specific specializations?"

	
	^ClosureCompiler
		ifNil: [self class compilerClass]
		ifNotNil: [ClosureCompiler]! !

!SymbiosisTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
constructBlock
	
	self subclassResponsibility! !

!SymbiosisTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
content
	"This method returns the internal content, as it was parsed"

	
	^content! !

!SymbiosisTerm methodsFor: 'access' stamp: 'AndyKellens 7/30/2012 17:06'!
evaluateIn: anEnv startAt: anIndex
	"we need to evaluate the block for this term. unbound variables should be  
	looked up in the environment"

	
	| result |
	variables at: 1 put: anEnv.
	variables at: 2 put: anIndex.
	result := [block valueWithArguments: variables]
		on: UnboundVariableException
		do: [:ex | self].
	variables at: 1 put: nil.
	variables at: 2 put: nil.
	^result! !

!SymbiosisTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToSymbiosisTerm: self! !

!SymbiosisTerm methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToSymbiosisTerm: aSymbiosisTerm
	
	| val1 val2 |
	"we dont allow them as we can not store them efficiently:
	we either store them with environment + index in the tables (but the env's will be popped/destroyed, so that won't work),
	and we have to constantly evaluate them again in order to get the value"

	self halt:
			'you are using symbiosisterms with tabling... there be dragons here'.
	val1 := aSymbiosisTerm evaluateIn: nil startAt: nil.
	val2 := self evaluateIn: nil startAt: nil.
	^val1 = val2! !

!SymbiosisTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
parse
	
	self subclassResponsibility! !

!SymbiosisTerm methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
parseVariables
	
	self subclassResponsibility! !

!SymbiosisTerm methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:02'!
prettyPrintOn: aStream scope: id
	
	| inVar attribute |

	inVar := false.
	attribute := TextColor black.
	source
		do:
			[:char | 
			"this is ugly... but at least it works fast :-)"

			inVar
				ifTrue:
					[char isAlphaNumeric | char isSymbol
						ifFalse:
							[inVar := false.
							attribute := TextColor black.]].
			char = $? | (char = $&)
				ifTrue:
					[attribute := TextColor color: (Color r:0 g:0 b:256).
					inVar := true].
			aStream withAttribute: attribute do:[aStream write: char]].
! !

!SymbiosisTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
source
	"This method returns the source code that is encapsulated by the symbiosisterm."

	
	^source! !

!SymbiosisTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
source: aString
	
	source := aString.
	self parse.
	self parseVariables.
	self constructBlock! !

!SymbiosisTerm methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
source: aString variables: aVarCollection content: aContent block: aBlock
	
	content := aContent.
	variables := aVarCollection.
	block := aBlock.
	source := aString! !

!SymbiosisTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
variables
	"the internal representation of these variables is a collection with nil's on the first two places. They are used for evaluation purposes. This method returns the list of 	variables, to be used for (possible) external usage"

	
	^variables
		copyFrom: 3
		to: variables size! !

!SymbiosisTerm methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
variablesIncludingNils
	
	^variables! !

ResolvableTerm subclass: #UppedObject
	instanceVariableNames: 'smalltalkValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!UppedObject commentStamp: '<historical>' prior: 0!
This class is used to represent smalltalk objects in a logic program (and in the logic parsetree)
!

UppedObject subclass: #Constant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!Constant commentStamp: '<historical>' prior: 0!
This class represents simple logic constants. In SOUL these are actually the same as Smalltalk Symbols (wrapped in Upped Objects). This class specializes the upped Smalltalk symbols a little for pretty printing and some other stuff

!

!Constant class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
name: aSymbol
	
	^self new setName: aSymbol! !

!Constant methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor constantVisit: self! !

!Constant methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: anIndex
	"see comment in AbstractTerm"

	
	^self name asString! !

!Constant methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
functor
	
	^self! !

!Constant methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isConstantFunctor
	
	^true! !

!Constant methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
name
	"return the name of the constant"

	
	^smalltalkValue! !

!Constant methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
predicate
	"see comment in AbstractTerm"

	
	^self asPredicateNameInEnv: nil startAt: nil! !

!Constant methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
predicateInEnv: anEnv startAt: anIndex
	"see comment in AbstractTerm"

	
	^self asPredicateNameInEnv: anEnv startAt: anIndex! !

!Constant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	self smalltalkValue printAsPrologOn: aStream scope: id in: env startAt:
			idx	"smalltalkValue printAsPrologOn: aStream scope: id"! !

!Constant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	| x name |
	aStream write: self class name , ' name: #'''.	"I should first convert every quote into a double quote"

	name := smalltalkValue asString.
	1
		to: name size
		do:
			[:i | 
			aStream write: (x := name at: i).
			x == $' ifTrue: [aStream write: x]].
	aStream write: ''''! !

!Constant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	| stringRepresentation |
	stringRepresentation := smalltalkValue asString.
	(stringRepresentation includes: $:) ifTrue: [aStream write: '#'].
	aStream write: smalltalkValue asString! !

!Constant methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: env startAt: anIndex
	"resolve as a normal term: do lookup in repository and create a resolver  on it"

	
	| foundClauses |
	foundClauses := self definitionRepository
		normalAddressLookup:
			(self
				lexicalAddressInEnv: env
				startAt: anIndex
				ifAbsent:
					[^self
						clauseNotFoundIn: self definitionRepository
						inEnv: env
						startAt: anIndex])
		multiplicity: 0.
	^SoulFactory current makeResolver fromClause: self toClauses: foundClauses
		inEnv: env startAt: anIndex! !

!Constant methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setName: aSymbol
	"initialize a constant. aSymbol can also contain an integer, in that case we must make it an integer"

	
	aSymbol isAllDigits
		ifTrue: [self setSmalltalkValue: aSymbol asNumber asInteger]
		ifFalse: [self setSmalltalkValue: aSymbol]! !

!Constant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!Constant methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	"See comment in AbstractTerm"

	
	self name isString
		ifTrue: [^self predicate = aPredicate & (self multiplicity = aNumber)].
	^false! !

Constant subclass: #EmptyListConstant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!EmptyListConstant commentStamp: '<historical>' prior: 0!
An EmptyListConstant is the special kind of constant to denote empty (logic) lists!

!EmptyListConstant class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^super new setName: #<>! !

!EmptyListConstant methodsFor: 'comparing' stamp: 'sp 24/10/11 15:26'!
= anObject
	
	^anObject species == EmptyListConstant! !

!EmptyListConstant methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor emptyListConstantVisit: self! !

!EmptyListConstant methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConditionFor: counter
	
	| tokens1var tokens2var terminals |
	terminals := OrderedCollection new.
	self listDo: [:aTerm | terminals add: aTerm].
	tokens1var := SoulFactory current makeVariable
		name: 'tokens' , counter asString.
	tokens2var := SoulFactory current makeVariable
		name: 'tokens' , (counter + 1) asString.
	^SoulFactory current makeCompound
		functor: (Constant name: #equals)
		termSequence:
			(TermSequence terms: (OrderedCollection with: tokens1var with: tokens2var))! !

!EmptyListConstant methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
collectIn: anEnv startAt: anIndex
	
	^Set new! !

!EmptyListConstant methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isEmpty
	
	^true! !

!EmptyListConstant methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToEmptyList: self! !

!EmptyListConstant methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToEmptyList: anEmptyListConstant
	
	^true! !

!EmptyListConstant methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
listDo: aBlock
	"see comment in ListTerm"
	
	! !

!EmptyListConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"this method is called if we are printed inside of a list (we are an 
	emptylistconstant, so we should not be printed)"
	
	! !

!EmptyListConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyListPrintOn: aStream scope: id
	"this method is called if we are printed inside of a list (we are an 
	emptylistconstant, so we should not be printed)"
	
	! !

!EmptyListConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologListOn: aStream scope: id in: env startAt: idx
	"this method is called if we are printed inside of a list (we are an 
	emptylistconstant, so we should not be printed)"
	
	! !

!EmptyListConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	aStream write: '[]'! !

!EmptyListConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' new'! !

!EmptyListConstant methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"Down of an emptyList is the emptyList"

	
	^self! !

!EmptyListConstant methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	"See comment in AbstractTerm"

	
	^self! !

!EmptyListConstant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithEmptyListConstant: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

!EmptyListConstant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^true! !

!EmptyListConstant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: hisIndex
		hisIndex: myIndex inSource: inSource! !

!EmptyListConstant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anObject soulDownIn: anEnv startAt: hisIndex.
	^((val isKindOf: SequenceableCollection)
		and: [(val isKindOf: CharacterArray) not])
		ifTrue: [^val isEmpty]
		ifFalse:
			[^super unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex
				hisIndex: hisIndex inSource: inSource]! !

!EmptyListConstant methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithEmptyListConstant: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

Constant subclass: #QuotedCodeConstant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!QuotedCodeConstant methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	
	^self asString! !

!QuotedCodeConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	| x name |
	aStream write: self class name , ' name: '''.	"I should first convert every quote into a double quote"

	name := smalltalkValue asString.
	1
		to: name size
		do:
			[:i | 
			aStream write: (x := name at: i).
			x == $' ifTrue: [aStream write: x]].
	aStream write: ''''! !

!QuotedCodeConstant methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	aStream write: smalltalkValue asString! !

!QuotedCodeConstant methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setName: aName
	
	self setSmalltalkValue: aName! !

!QuotedCodeConstant methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockString
	
	^'''' , self asString , ''''! !

!QuotedCodeConstant methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	
	^self! !

!QuotedCodeConstant methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self! !

UppedObject subclass: #UppedASTObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithUppedASTObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	(aCompound isASTLinkedIn: anEnv startAt: targetIndex)
		ifTrue:
			[^self
				unifyWith: aCompound astLink
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource]
		ifFalse:
			[^(self
				unifyWith: aCompound astLink
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource)
				and:
					[smalltalkValue unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex:
							sourceIndex hisIndex: targetIndex inSource: inSource]]! !

!UppedASTObject methodsFor: 'unification' stamp: 'AndyKellens 3/9/2012 14:25'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithPosVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithUppedASTObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^smalltalkValue source
		matchesRegex: (aQCTerm asSourceStringIn: anEnv startAt: targetIndex)! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^smalltalkValue
		unifyWith: anObject smalltalkValue
		inEnv: anEnv
		myIndex: sourceIndex
		hisIndex: targetIndex
		inSource: inSource! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	(aCompound isASTLinkedIn: anEnv startAt: targetIndex)
		ifTrue:
			[^self
				unifyWith: aCompound astLink
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource]
		ifFalse:
			[^(self
				unifyWith: aCompound astLink
				inEnv: anEnv
				myIndex: sourceIndex
				hisIndex: targetIndex
				inSource: inSource)
				and:
					[smalltalkValue unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv
						myIndex: sourceIndex hisIndex: targetIndex inSource: inSource]]! !

!UppedASTObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithUppedASTObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UppedObject class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
up: anObject
	
	^self new setSmalltalkValue: anObject! !

!UppedObject methodsFor: 'comparing' stamp: 'sp 24/10/11 15:26'!
= anObject
	
	^(anObject isKindOf: UppedObject)
		ifTrue: [anObject smalltalkValue = self smalltalkValue]
		ifFalse: [super = anObject]! !

!UppedObject methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor objectVisit: self! !

!UppedObject methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConclusionWithSize: aSize
	
	^SoulFactory current makeCompound
		functor: self
		termSequence:
			(TermSequence
				terms:
					(OrderedCollection
						with: (SoulFactory current makeVariable name: 'tokens1')
						with: (SoulFactory current makeVariable name: 'tokens' , aSize asString)))! !

!UppedObject methodsFor: 'converting' stamp: 'AndyKellens 11/2/2011 09:53'!
asDCGConditionFor: counter
	
	| terms |
	terms := OrderedCollection new.
	terms
		add: (SoulFactory current makeVariable name: 'tokens' , counter asString).
	terms
		add:
			(SoulFactory current makeVariable name: 'tokens' , (counter + 1) asString).
	^SoulFactory current makeCompound
		functor: self
		termSequence: (TermSequence terms: terms)! !

!UppedObject methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: anIndex
	
	^smalltalkValue asString! !

!UppedObject methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	
	^smalltalkValue asSourceString! !

!UppedObject methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
displayString
	
	^smalltalkValue displayString! !

!UppedObject methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToUppedObject: self! !

!UppedObject methodsFor: 'slgtesting' stamp: 'AndyKellens 11/3/2011 11:19'!
isSLGEquivalentToUppedObject: anUppedObject
	^anUppedObject = self! !

!UppedObject methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUppedObject
	
	^true! !

!UppedObject methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"this method is called if we are printed inside a list"

	
	aStream write: ','.
	self printOn: aStream! !

!UppedObject methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:07'!
prettyPrintAsFunctorOn: aStream scope: id
	
	id = #asClause
		ifTrue:
			[aStream withAttribute:(TextEmphasis bold) do:[aStream write: smalltalkValue asString]] ifFalse:[aStream write: smalltalkValue asString.]! !

!UppedObject methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:07'!
prettyPrintAsKeywordOn: aStream scope: id
	
	id = #asClause
		ifTrue:
			[aStream withAttribute:(TextEmphasis bold) do:[aStream write: smalltalkValue asString]] ifFalse:[aStream write: smalltalkValue asString.]! !

!UppedObject methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:07'!
printAsFunctorOn: aStream
	
	aStream nextPutAll: smalltalkValue asString! !

!UppedObject methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologListOn: aStream scope: scope in: env startAt: idx
	"this method is called if we are printed inside a list"

	
	aStream write: ','.
	self printAsPrologOn: aStream scope: scope in: env startAt: idx! !

!UppedObject methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	smalltalkValue printAsPrologOn: aStream scope: id in: env startAt: idx! !

!UppedObject methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in QSOULAbstractTerm"

	
	aStream write: '['.
	smalltalkValue printOn: aStream.
	aStream write: ']'! !

!UppedObject methodsFor: 'resolving' stamp: 'AndyKellens 11/2/2011 09:53'!
resolveIn: anEnv startAt: anIndex
	
	| resolveBlock resolveBlockArgs |
	resolveBlockArgs := OrderedCollection new.
	resolveBlockArgs add: nil.
	resolveBlock := [:arg | 
	(self smalltalkValue isKindOf: Boolean)
		ifFalse:
			[self error: 'SmallTalkClauses should always evaluate to true or false!!'].
	self smalltalkValue
		ifTrue:
			[ResolutionResult
				condition: SoulFactory current makeTermSequence new
				rollbackInfo: anEnv rollbackInfo
				conditionsEnvIndex: nil
				cutState: false]].
	^NativeResolver onBlock: resolveBlock withArguments: resolveBlockArgs! !

!UppedObject methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setSmalltalkValue: anObject
	
	smalltalkValue := anObject! !

!UppedObject methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
smalltalkValue
	
	^smalltalkValue! !

!UppedObject methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"downing an uppedObject to the SmallTalk environment is returning my  
	wrapped value"

	
	^smalltalkValue! !

!UppedObject methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithUppedObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^(smalltalkValue isKindOf: SequenceableCollection)
		and: [smalltalkValue isEmpty]! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithKeywordFunctor: term inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self smalltalkValue asString = term asString! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| list |
	^((smalltalkValue isKindOf: SequenceableCollection)
		and: [(smalltalkValue isKindOf: CharacterArray) not])
		ifTrue:
			[list := ListTerm newFromCollection: smalltalkValue.
			list unifyWithListTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex:
					hisIndex inSource: inSource]
		ifFalse: [false]! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithMessageFunctor: term inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^self smalltalkValue asString = term asString! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithUppedObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: inSource not! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a SmallTalkTerm if the result of this smalltalkterm unifies 
	with me"

	
	^self
		unifyWithUppedObject:
			(anSTTerm soulDownIn: anEnv startAt: hisIndex) soulUp
		inEnv: anEnv
		myIndex: myIndex
		hisIndex: hisIndex
		inSource: inSource! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with another object if my downed versions are equal"

	
	^(self soulDownIn: anEnv startAt: myIndex)
		= (anObject soulDownIn: anEnv startAt: hisIndex)! !

!UppedObject methodsFor: 'unification' stamp: 'AndyKellens 10/25/2011 11:01'!
unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	| list |
	^((smalltalkValue isKindOf: SequenceableCollection)
		and: [(smalltalkValue isKindOf: String) not])
		ifTrue:
			[list := ListTerm newFromCollection: smalltalkValue.
			list unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex:
					sourceIndex hisIndex: targetIndex inSource: inSource]
		ifFalse: [false]! !

!UppedObject methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"we group all unfication with variables in the Variable class"

	
	^aVariable
		unifyWithUppedObject: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UppedObject methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	
	^false! !

AbstractTerm subclass: #TermSequence
	instanceVariableNames: 'collection containsCut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!TermSequence commentStamp: '<historical>' prior: 0!
This class is used to represent sequences of terms inside compoundterms, lists, etc...

Instance Variables:
	collection	<(LinkedList of: AbstractTerm) | (List of: AbstractTerm) | (OrderedCollection of: AbstractTerm) | (RunArray of: AbstractTerm) | (SlidingInterval of: AbstractTerm)>	description of collection
	containsCut	<Boolean>	description of containsCut

!

TermSequence subclass: #ProgramSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!ProgramSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
clauses: aCollection
	
	^self terms: aCollection! !

!ProgramSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
addAll: programSequence
	
	collection addAll: programSequence clauses! !

!ProgramSequence methodsFor: 'repository' stamp: 'sp 24/10/11 15:26'!
addToRepository: aRepository
	
	self clauses do: [:c | c addToRepository: aRepository]! !

!ProgramSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
clauses
	
	^collection! !

!ProgramSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	
	aStream write: '^(' , self class name , ' terms: (OrderedCollection new '.
	collection
		do:
			[:currentClause | 
			aStream write: 'add:('.
			currentClause printForCompileOn: aStream.
			aStream write: ')']
		separatedBy: [aStream write: ';'].
	aStream write: ';yourself))'! !

!ProgramSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	collection
		do: [:currentClause | currentClause printOn: aStream]
		separatedBy:
			[aStream
				write: '.';
				cr]! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^self terms: OrderedCollection new! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
term: term
	
	^self terms: (OrderedCollection with: term)! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
term: term1 term: term2
	
	^self terms: (OrderedCollection with: term1 with: term2)! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
term: term1 term: term2 term: term3
	
	^self terms: (OrderedCollection with: term1 with: term2 with: term3)! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
term: term1 term: term2 term: term3 term: term4
	
	^self
		terms:
			(OrderedCollection with: term1 with: term2 with: term3 with: term4)! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
terms: aCollection
	
	^self terms: aCollection containsCut: false! !

!TermSequence class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
terms: aCollection containsCut: boolean
	
	^super new setTerms: aCollection containsCut: boolean! !

!TermSequence methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor termSequenceVisit: self! !

!TermSequence methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addChildrenFirst: aChild
	"used by the parser"

	
	^collection addFirst: aChild first! !

!TermSequence methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addChildrenInitial: aCollection
	"used by the parser"

	
	collection := aCollection! !

!TermSequence methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
addTermFirst: aTerm
	"used by the parser"

	
	(aTerm isKindOf: Cut) ifTrue: [containsCut := true].
	self terms addFirst: aTerm! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
containsCut
	
	^containsCut! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
containsCut: boolean
	
	containsCut := boolean! !

!TermSequence methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
do: aBlock
	
	collection do: aBlock! !

!TermSequence methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
do: aBlock separatedBy: block2
	
	collection do: aBlock separatedBy: block2! !

!TermSequence methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToTermSequence: self! !

!TermSequence methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToTermSequence: aTermSequence
	
	| myTerms otherTerms otherTerm |
	myTerms := self terms.
	otherTerms := aTermSequence terms.
	myTerms size = otherTerms size ifFalse: [^false].
	myTerms
		doWithIndex:
			[:term :index | 
			otherTerm := otherTerms at: index.
			(term isSLGEquivalentTo: otherTerm) ifFalse: [^false]].	"even when all the individual terms match we still need to check cases like this:
	path(?X,?Y) vs path(?X,?X). The first one gives every path, the latter should return cycles"

	^(aTermSequence isVariantOf: self) and: [self isVariantOf: aTermSequence]! !

!TermSequence methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isVariantOf: aTermSequence
	"we assume that both sequences are already SLGEquivalent"

	
	| myTerms otherTerms otherTerm dict |
	myTerms := self terms.
	otherTerms := aTermSequence terms.
	dict := Dictionary new.
	myTerms
		doWithIndex:
			[:term :index | 
			otherTerm := otherTerms at: index.
			term isVariable
				ifTrue:
					[dict
						at: term name
						ifAbsent:
							[dict
								at: term name
								put: otherTerm name].
					dict
						at: term name
						ifPresent: [:name | name = otherTerm name ifFalse: [^false]]]].
	^true! !

!TermSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"

	
	id = #asClause
		ifTrue:
			[aStream tab.
			collection
				do: [:term | term prettyPrintOn: aStream scope: #asClause]
				separatedBy:
					[aStream write: ','.
					aStream crtab]]
		ifFalse:
			[collection
				do: [:term | term prettyPrintOn: aStream scope: #asTerm]
				separatedBy: [aStream write: ',']]! !

!TermSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: id in: env startAt: idx
	
	id = #asClause
		ifTrue:
			[collection
				do:
					[:term | term printAsPrologOn: aStream scope: #asClause in: env startAt: idx]
				separatedBy:
					[aStream write: ','.
					aStream crtab]]
		ifFalse:
			[collection
				do:
					[:term | term printAsPrologOn: aStream scope: #asTerm in: nil startAt: nil]
				separatedBy: [aStream write: ',']]! !

!TermSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' terms: (OrderedCollection new '.
	self terms
		do:
			[:currentClause | 
			aStream write: 'add:('.
			currentClause printForCompileOn: aStream.
			aStream write: ')']
		separatedBy: [aStream write: ';'].
	aStream write: ';yourself) containsCut: ' , self containsCut asString! !

!TermSequence methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	collection
		do: [:term | term printOn: aStream]
		separatedBy: [aStream write: ',']! !

!TermSequence methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: aRepository
	
	self definitionRepository: aRepository.
	collection do: [:each | each recursiveDefinitionRepository: aRepository]! !

!TermSequence methodsFor: 'enumeration' stamp: 'sp 24/10/11 15:26'!
reverseDo: aBlock
	
	collection reverseDo: aBlock! !

!TermSequence methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setTerms: aCollection containsCut: aBoolean
	
	collection := aCollection.
	containsCut := aBoolean! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
size
	
	^collection size! !

!TermSequence methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	
	^TermSequence
		terms:
			(collection
				collect:
					[:aSubTerm | (aSubTerm soulDownIn: anEnv startAt: anIndex) soulUp])
		containsCut: containsCut! !

!TermSequence methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulUp: anEnv
	
	^self! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
terms
	
	^collection! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
terms: aCollection
	
	collection := aCollection! !

!TermSequence methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	| newSubTerms |
	newSubTerms := OrderedCollection new.
	self
		do:
			[:aTerm | newSubTerms add: (aTerm transitiveLookupIn: anEnv startAt: anIndex)].
	^self class
		terms: newSubTerms
		containsCut: self containsCut! !

!TermSequence methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithTermSequence: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!TermSequence methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithTermSequence: other inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"transfered"

	
	collection size = other terms size ifFalse: [^false].
	collection
		with: other terms
		do:
			[:myTerm :otherTerm | 
			(myTerm unifyWith: otherTerm inEnv: anEnv myIndex: myIndex hisIndex:
					hisIndex inSource: inSource) ifFalse: [^false]].
	^true! !

!TermSequence methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
usesPredicate: aPredicate multiplicity: aNumber
	
	collection
		do:
			[:aTerm | (aTerm usesPredicate: aPredicate multiplicity: aNumber) ifTrue: [^true]].
	^false! !

AbstractTerm subclass: #UnaryMessageFunctor
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!UnaryMessageFunctor class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
message: message
	
	^self new message: message! !

!UnaryMessageFunctor class methodsFor: 'instance creation' stamp: 'AndyKellens 11/2/2011 09:53'!
name: aName
	
	^self message: (SoulFactory current makeConstant name: aName)! !

!UnaryMessageFunctor methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	
	^aVisitor unaryMessageFunctorVisit: self! !

!UnaryMessageFunctor methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: idx
	
	^message asPredicateNameInEnv: anEnv startAt: idx! !

!UnaryMessageFunctor methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isUnaryMessageFunctor
	
	^true! !

!UnaryMessageFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
message
	
	^message! !

!UnaryMessageFunctor methodsFor: 'initialization' stamp: 'sp 24/10/11 15:26'!
message: msg
	
	message := msg! !

!UnaryMessageFunctor methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
messageSelector
	
	^message name asSymbol! !

!UnaryMessageFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
multiplicity
	
	^1! !

!UnaryMessageFunctor methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
name
	
	^self message name! !

!UnaryMessageFunctor methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:03'!
prettyPrintOn: aStream scope: id
	
	| temp attributes|
	attributes := OrderedCollection with:(TextColor black).
	id = #asClause
		ifTrue:
			[attributes add:TextEmphasis bold].
	aStream withAttributes: attributes do: [self printOn: aStream].
	! !

!UnaryMessageFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologFunctorOn: aStream scope: id in: env startAt: idx
	"TODO: What if it is a variable?"

	
	aStream write: self name! !

!UnaryMessageFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	
	aStream write: self class name , ' message: ('.
	message printForCompileOn: aStream.
	aStream write: ')'! !

!UnaryMessageFunctor methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	
	^message printOn: aStream! !

!UnaryMessageFunctor methodsFor: 'scoping' stamp: 'sp 24/10/11 15:26'!
recursiveDefinitionRepository: repository
	
	super recursiveDefinitionRepository: repository.
	message recursiveDefinitionRepository: repository! !

!UnaryMessageFunctor methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self class message: (message transitiveLookupIn: anEnv startAt: anIndex)! !

!UnaryMessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithUnaryMessageFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UnaryMessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUnaryMessageFunctor: unaryFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"transfered"

	
	^self message
		unifyWith: unaryFunctor message
		inEnv: anEnv
		myIndex: myIndex
		hisIndex: hisIndex
		inSource: inSource! !

!UnaryMessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithUnaryMessageFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!UnaryMessageFunctor methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^aTerm
		unifyWithUnaryMessageFunctor: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

AbstractTerm subclass: #Variable
	instanceVariableNames: 'address name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!Variable commentStamp: '<historical>' prior: 0!
Representation of a variable inside a logic parsetree.

Variables have a name and an address. The address is determined at 'parse time' and it is relative to the start of the activation record of the logic rule they are written in. 
!

Variable subclass: #DecoratedVariable
	instanceVariableNames: 'decorators'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!DecoratedVariable commentStamp: '<historical>' prior: 0!
This class represents a variable that was decorated with additional types.

Instance Variables:
	decorators	<(LinkedWeakAssociationDictionary of: (AbstractTerm | Fact | Query | Rule)) | (SequenceableCollection of: (AbstractTerm | Fact | Query | Rule)) | TermSequence>	description of decorators

!

!DecoratedVariable methodsFor: 'decorators' stamp: 'sp 24/10/11 15:26'!
decorators: aCollection
	
	decorators := aCollection! !

!DecoratedVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: '('.
	super printForCompileOn: aStream.
	aStream write: ') decorators: (OrderedCollection new '.
	decorators
		do:
			[:each | 
			aStream write: 'add:('.
			each printForCompileOn: aStream.
			aStream write: ')']
		separatedBy: [aStream write: ';'].
	aStream write: ';yourself)'! !

!DecoratedVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	
	super printOn: aStream.
	aStream nextPutAll: '\'.
	decorators
		do: [:each | each printOn: aStream]
		separatedBy: [aStream nextPutAll: '\']! !

!DecoratedVariable methodsFor: 'quotedCode' stamp: 'AndyKellens 11/2/2011 09:53'!
quotedCodeToList
	
	| previous |
	previous := self asNormalVariable.
	decorators
		reverseDo:
			[:each | 
			previous := SoulFactory current makeCompound
				functor: each
				termSequence:
					(SoulFactory current makeTermSequence terms: (OrderedCollection with: previous))].
	^previous! !

Variable subclass: #DelayedVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!DelayedVariable commentStamp: '<historical>' prior: 0!
Represents a special kind of variables. Delayed variables have the special meaning that any Smalltalk Term unified with them will not be evaluated but merely bound to them. Normal variables evaluate the Smalltalk Term and bind to the resulting value.

!

!DelayedVariable methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor delayedVariableVisit: self! !

!DelayedVariable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asSmalltalkString
	"see comment in AbstractTerm"

	
	^'(env transitiveLookup: soulvar' , self name , ')'! !

!DelayedVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"this method is called if we are printed inside a list"

	
	aStream write: '|'.
	self printOn: aStream! !

!DelayedVariable methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:04'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"

	
	aStream withAttribute:(TextColor red) do:[
	aStream write: '&' , name].
	! !

!DelayedVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' name: '''.
	aStream write: name.
	aStream write: ''''! !

!DelayedVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	"aStream write: '&' , name , ':' , address asString"

	
	aStream write: '&' , name! !

!DelayedVariable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockDowningString
	"see comment in Variable"

	
	^'(env transitiveLookup: arg' , self name , ' startAt: envIndex)'! !

!DelayedVariable methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	
	| value |
	value := anEnv lookup: self startAt: anIndex.
	value isNil
		ifTrue: [^self]
		ifFalse:
			[^value term
				transitiveLookupIn: anEnv
				startAt: value envIndex]! !

!DelayedVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: aBool
	
	^aTerm
		unifyWithDelayedVariable: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: aBool not! !

!DelayedVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anSTTerm startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithSmalltalkTerm: anSTTerm
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!DelayedVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	| val val2 |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[val2 := anEnv lookup: aVariable startAt: hisIndex.
			val2 isNil
				ifTrue:
					[inSource
						ifTrue:
							[anEnv bind: self to: aVariable startAt: myIndex valueStartAt: hisIndex]
						ifFalse:
							[anEnv bind: aVariable to: self startAt: hisIndex valueStartAt: myIndex].
					^true]
				ifFalse:
					[^val2 term
						unifyWithDelayedVariable: self
						inEnv: anEnv
						myIndex: val2 envIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]]
		ifFalse:
			[^val term
				unifyWithDelayedVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

Variable subclass: #HiddenVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!HiddenVariable methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor hiddenVariableVisit: self! !

!HiddenVariable methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isHiddenVariable
	
	^true! !

Variable subclass: #NegVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!NegVariable methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:04'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"
	aStream withAttribute: (TextColor red) do: [aStream write: '-?' , name]
	! !

!NegVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	"aStream write: '&' , name , ':' , address asString"

	
	aStream write: '-?' , name! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithNegVariable: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithResultMessageTerm: aResultMessage inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithTemplateQueryTerm: aTQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!NegVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val val2 |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[val2 := anEnv lookup: aVariable startAt: hisIndex.
			val2 isNil
				ifTrue:
					[aVariable ~~ self
						ifTrue:
							[inSource
								ifTrue:
									[anEnv
										bind: self asNormalVariable
										to: aVariable
										startAt: myIndex
										valueStartAt: hisIndex]
								ifFalse:
									[anEnv
										bind: aVariable
										to: self asNormalVariable
										startAt: hisIndex
										valueStartAt: myIndex]].
					^true]
				ifFalse:
					[^val2 term
						unifyWithNegVariable: self
						inEnv: anEnv
						myIndex: val2 envIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]]
		ifFalse:
			[^val term
				unifyWithVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

Variable subclass: #PosVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!PosVariable methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:04'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"
	aStream withAttribute: (TextColor red) do: [aStream write: '+?' , name]
	! !

!PosVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	"aStream write: '&' , name , ':' , address asString"

	
	aStream write: '+?' , name! !

!PosVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithPosVariable: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!PosVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUnderscoreVariable: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^false! !

!PosVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val val2 |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[val2 := anEnv lookup: aVariable startAt: hisIndex.
			val2 isNil
				ifTrue: [^false]
				ifFalse:
					[^val2 term
						unifyWithPosVariable: self
						inEnv: anEnv
						myIndex: val2 envIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]]
		ifFalse:
			[^val term
				unifyWithVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

Variable subclass: #UnderscoreVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-GrammarTerms'!

!UnderscoreVariable commentStamp: '<historical>' prior: 0!
This class represents the unnamed variable in logic parsetrees!

!UnderscoreVariable methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in QSOULAbstractTerm"

	
	^aVisitor underscoreVariableVisit: self! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
listPrintOn: aStream
	"Needed to display < ?avar  | ? > in SoulInteractive"

	
	aStream write: '|'.
	self printOn: aStream! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
prettyListPrintOn: aStream scope: id
	"Needed to display < ?avar  | ? > in SoulInteractive"

	
	aStream write: '|'.
	self printOn: aStream! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:04'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"

	
	aStream withAttribute: (TextColor red) do: [self printOn: aStream]! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologListOn: aStream scope: scope in: env startAt: idx
	"Needed to display < ?avar  | ? > in SoulInteractive"

	
	aStream write: '|'.
	self printAsPrologOn: aStream scope: scope in: env startAt: idx! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: anObject in: env startAt: idx
	
	aStream write: '_'! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' new'! !

!UnderscoreVariable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in QSOULAbstractTerm"

	"Why not simply write: $? instead of Character value:63... well, if the  
	QSOULparsetreegenerator wanted to parse this (because I request the  
	parsetree of this method), it would generate a fact containing '?'  
	somewhere. Later on, the QSOUL interpreter would see a variable...  
	very confusing indeed"

	
	aStream write: (Character value: 63)! !

!UnderscoreVariable methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	
	^self! !

!UnderscoreVariable methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	
	^self! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^aTerm unifyWithUnderscoreVariable: self inEnv: anEnv myIndex: myIndex
		hisIndex: hisIndex inSource: inSource! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aList inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithNegVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with negative variables"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithPosVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I do not unify with a positive variable"

	
	^false! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!UnderscoreVariable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with everything"

	
	^true! !

!Variable class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
name: aName
	
	^self new setName: aName! !

!Variable methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
accept: aVisitor
	"see comment in AbstractTerm"

	
	^aVisitor variableVisit: self! !

!Variable methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
address
	"a variable keeps its lexical address in the environment,  relative to a particular startindex of the activation record of the rule."

	
	^address! !

!Variable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asNormalVariable
	
	^(Variable name: self name) setAddress: address! !

!Variable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asPredicateNameInEnv: anEnv startAt: anIndex
	
	| val |
	"see comment in AbstractTerm"

	val := anEnv lookup: self startAt: anIndex.
	^val isNil
		ifTrue:
			[self error: 'Trying to resolve a term with an unbound variable as functor']
		ifFalse:
			[val term
				asPredicateNameInEnv: anEnv
				startAt: val envIndex]! !

!Variable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
asSourceStringIn: anEnv startAt: anIndex
	"see comment in AbstractTerm"

	"if I am bound, return the sourcestring of my bound value"

	
	| value |
	value := anEnv lookup: self startAt: anIndex.
	value isNil
		ifTrue: [^self asString]
		ifFalse:
			[^value term
				asSourceStringIn: anEnv
				startAt: value envIndex]! !

!Variable methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
capitalizedName
	
	^(name copy)
		at: 1
			put: name first asUppercase;
		yourself! !

!Variable methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isIndexable
	
	^false! !

!Variable methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentTo: aTerm
	
	^aTerm isSLGEquivalentToVariable: self! !

!Variable methodsFor: 'slgtesting' stamp: 'sp 24/10/11 15:26'!
isSLGEquivalentToVariable: aVariable
	
	^true! !

!Variable methodsFor: 'testing' stamp: 'sp 24/10/11 15:26'!
isVariable
	
	^true! !

!Variable methodsFor: 'list enumeration' stamp: 'sp 24/10/11 15:26'!
listDo: aBlock
	
	! !

!Variable methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
name
	"a variable has a name"

	
	^name! !

!Variable methodsFor: 'printing' stamp: 'AndyKellens 11/4/2011 15:03'!
prettyPrintOn: aStream scope: id
	"see comment in AbstractTerm"
	
	aStream withAttribute: (TextColor color:(Color r:0 g:0 b:256)) do: [
	aStream write: '?' , name].
	! !

!Variable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printAsPrologOn: aStream scope: anObject in: env startAt: idx
	
	| value |
	"TODO: Should use a dummy environment instead of nil values"

	(env isNil or: [(value := env lookup: self startAt: idx) isNil])
		ifTrue: [aStream write: self capitalizedName]
		ifFalse:
			[value term
				printAsPrologOn: aStream
				scope: anObject
				in: env
				startAt: value envIndex]! !

!Variable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printForCompileOn: aStream
	"see comment in AbstractTerm"

	
	aStream write: self class name , ' name: '''.
	aStream write: name.
	aStream write: ''''! !

!Variable methodsFor: 'printing' stamp: 'sp 24/10/11 15:26'!
printOn: aStream
	"see comment in AbstractTerm"

	"aStream write: '?' , name , ':' , address asString"

	
	aStream write: '?' , name! !

!Variable methodsFor: 'reification' stamp: 'AndyKellens 10/25/2011 10:54'!
protectedSoulDownIn: anEnv startAt: anIndex
	"my downed value or an error"

	
	| value |
	value := anEnv lookup: self startAt: anIndex.
	value isNil
		ifTrue: [UnboundVariableException signal]
		ifFalse:
			[^value term
				soulDownIn: anEnv
				startAt: value envIndex]! !

!Variable methodsFor: 'resolving' stamp: 'sp 24/10/11 15:26'!
resolveIn: anEnv startAt: anIndex
	"resolve my value"

	
	| val |
	val := anEnv lookup: self startAt: anIndex.
	^val isNil
		ifTrue: [self error: 'Trying to resolve an unbound variable']
		ifFalse:
			[val term
				resolveIn: anEnv
				startAt: val envIndex]! !

!Variable methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
setAddress: newAddress
	"this method sets the address of the variable. Can only be used by the evaluator!!"

	
	address := newAddress! !

!Variable methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
setName: aName
	
	name := aName! !

!Variable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockDowningString
	"see comment in AbstractTerm"

	
	^'(arg' , self name , ' protectedSoulDownIn: env startAt: envIndex)'! !

!Variable methodsFor: 'converting' stamp: 'sp 24/10/11 15:26'!
smalltalkBlockString
	"see comment in AbstractTerm"

	
	^'soulvar' , self name! !

!Variable methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
soulDownIn: anEnv startAt: anIndex
	"myself or my downed value"

	
	| value |
	value := anEnv lookup: self startAt: anIndex.
	value isNil
		ifTrue: [^self]
		ifFalse:
			[^value term
				soulDownIn: anEnv
				startAt: value envIndex]! !

!Variable methodsFor: 'reification' stamp: 'sp 24/10/11 15:26'!
transitiveLookupIn: anEnv startAt: anIndex
	"myself or my downed value"

	
	| value |
	value := anEnv lookup: self startAt: anIndex.
	(value isNil or: [value term == self])
		ifTrue: [^self]
		ifFalse:
			[^value term
				transitiveLookupIn: anEnv
				startAt: value envIndex]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWith: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"see comment in AbstractTerm"

	
	^aTerm
		unifyWithVariable: self
		inEnv: anEnv
		myIndex: hisIndex
		hisIndex: myIndex
		inSource: (inSource xor: true)! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithAnnotatedTerm: aTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a fuzzy term by binding to it or unifying my value with it"

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aTerm startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithAnnotatedTerm: aTerm
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a compound by binding to it or unifying my value with it"

	
	| val hisAst |
	(hisAst := aCompound astLink soulDownIn: anEnv startAt: hisIndex) class ==
			UppedASTObject
		ifTrue:
			[^self unifyWithUppedObject: hisAst inEnv: anEnv myIndex: myIndex hisIndex:
					hisIndex inSource: inSource]
		ifFalse:
			[val := anEnv lookup: self startAt: myIndex.
			val isNil
				ifTrue:
					[anEnv bind: self to: aCompound startAt: myIndex valueStartAt: hisIndex.
					^true]
				ifFalse:
					[^val term
						unifyWithCompoundTerm: aCompound
						inEnv: anEnv
						myIndex: val envIndex
						hisIndex: hisIndex
						inSource: inSource]]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithCut: aCut inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	"I unify with a cut by binding to it or by unifying it with my 
	value "

	
	| val |
	val := anEnv lookup: self startAt: sourceIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aCut startAt: sourceIndex valueStartAt: targetIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithCut: aCut
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: targetIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithEmptyListConstant: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object by binding to it or by unifying it with my 
	value "

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anObject startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithEmptyListConstant: anObject
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithKeywordFunctor: anMPFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anMPFunctor startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithKeywordFunctor: anMPFunctor
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithListTerm: aList inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a compound by binding to it or unifying my value with it"

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aList startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithListTerm: aList
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithMessageFunctor: anMPFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anMPFunctor startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithMessageFunctor: anMPFunctor
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithNativeClause: aNative inEnv: anEnv myIndex: sourceIndex hisIndex: targetIndex inSource: inSource
	
	^self unifyWithUppedObject: aNative inEnv: anEnv myIndex: sourceIndex
		hisIndex: targetIndex inSource: inSource! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithNegVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val val2 |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[val2 := anEnv lookup: aVariable startAt: hisIndex.
			val2 isNil
				ifTrue:
					[aVariable ~~ self
						ifTrue:
							[inSource
								ifTrue:
									[anEnv
										bind: self
										to: aVariable asNormalVariable
										startAt: myIndex
										valueStartAt: hisIndex]
								ifFalse:
									[anEnv
										bind: aVariable asNormalVariable
										to: self
										startAt: hisIndex
										valueStartAt: myIndex]].
					^true]
				ifFalse:
					[^val2 term
						unifyWithVariable: self
						inEnv: anEnv
						myIndex: val2 envIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]]
		ifFalse:
			[^val term
				unifyWithNegVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithPosVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	"transferred"

	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue: [^false]
		ifFalse:
			[^val term
				unifyWithPosVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithQuotedCodeTerm: aQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aQCTerm startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithQuotedCodeTerm: aQCTerm
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithResultMessageTerm: aResultMessage inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aResultMessage startAt: myIndex valueStartAt:
					hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithResultMessageTerm: aResultMessage
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithSmalltalkTerm: anSTTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv
				bind: self
				to: (anSTTerm evaluateIn: anEnv startAt: hisIndex) soulUp
				startAt: myIndex
				valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithSmalltalkTerm: anSTTerm
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithTemplateQueryTerm: aTQCTerm inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self.
	val isNil
		ifTrue:
			[anEnv bind: self to: aTQCTerm startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithTemplateQueryTerm: aTQCTerm
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUnaryMessageFunctor: unaryMessageFunctor inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: unaryMessageFunctor startAt: myIndex valueStartAt:
					hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithUnaryMessageFunctor: unaryMessageFunctor
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedASTObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object by binding to it or by unifying it with my 
	value "

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anObject startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithUppedASTObject: anObject
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithUppedObject: anObject inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with an object by binding to it or by unifying it with my 
	value "

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: anObject startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithUppedObject: anObject
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVarArgsCompoundTerm: aCompound inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	"I unify with a compound by binding to it or unifying my value with it"

	
	| val |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[anEnv bind: self to: aCompound startAt: myIndex valueStartAt: hisIndex.
			^true]
		ifFalse:
			[^val term
				unifyWithVarArgsCompoundTerm: aCompound
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !

!Variable methodsFor: 'unification' stamp: 'sp 24/10/11 15:26'!
unifyWithVariable: aVariable inEnv: anEnv myIndex: myIndex hisIndex: hisIndex inSource: inSource
	
	| val val2 |
	val := anEnv lookup: self startAt: myIndex.
	val isNil
		ifTrue:
			[val2 := anEnv lookup: aVariable startAt: hisIndex.
			val2 isNil
				ifTrue:
					[(aVariable name = self name and: [myIndex = hisIndex])
						ifFalse:
							[inSource
								ifFalse:
									[anEnv bind: self to: aVariable startAt: myIndex valueStartAt: hisIndex]
								ifTrue:
									[anEnv bind: aVariable to: self startAt: hisIndex valueStartAt: myIndex]].
					^true]
				ifFalse:
					[^val2 term
						unifyWithVariable: self
						inEnv: anEnv
						myIndex: val2 envIndex
						hisIndex: myIndex
						inSource: (inSource xor: true)]]
		ifFalse:
			[^val term
				unifyWithVariable: aVariable
				inEnv: anEnv
				myIndex: val envIndex
				hisIndex: hisIndex
				inSource: inSource]! !
